from core import models
from core.translation import _t

class Action(models.Action):
    """Resize an image"""
    
    label       = _t('Image size')
    author      = 'Stani'
    email       = 'spe.stani.be@gmail.com'
    version     = '0.1'
    tags        = [_t('image')]
    __doc__     = _t('Scale an image with different filters')
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Width')]                  = models.PixelField('100%')
        fields[_t('Height')]                 = models.PixelField('100%')
        fields[_t('Resolution')]             = models.DpiField('<%s>'%_t('dpi'))
        fields[_t('Constrain Proportions')]  = models.BooleanField(True)
        fields[_t('Resample Image')]         = models.ImageResampleField('bicubic')
        
        super(Action,self).__init__(fields)
        
    def import_modules(self):
        global Image
        import Image
        
    def apply(self,photo,setting,cache):
        #get info
        info                = photo.get_info()
        #dpi
        dpi                 = self.get_field_value('Resolution',info)
        #size
        info                = photo.get_info()
        x0, y0              = info['new_size']
        x1                  = self.get_field_size('Width',info,x0,dpi)
        y1                  = self.get_field_size('Height',info,y0,dpi)
        #check if image size has changed
        if x0!=x1 or y0!=y1:
            proportions     = self.get_field_value('Resolution',info)
            method          = self.get_field_value('Resample Image',info)
            method          = getattr(Image,method)
            if proportions:
                scaleX      = float(x1)/x0
                scaleY      = float(y1)/y0
                if scaleX < scaleY:
                    y1      = int(round(y0*scaleX))
                else:
                    x1      = int(round(x0*scaleY))
            photo.resize((x1,y1),method)
        return photo
    
    description = """
This action offers four different resampling filters.
- NEAREST. Pick the nearest pixel from the input image. Ignore all other input
  pixels.
- BILINEAR. Use linear interpolation over a 2x2 environment in the input image.
  Note that in the current version of PIL, this filter uses a fixed input environment
  when downsampling.
- BICUBIC. Use cubic interpolation over a 4x4 environment in the input image.
  Note that in the current version of PIL, this filter uses a fixed input environment
  when downsampling.
- ANTIALIAS. Calculate the output pixel value using a
  high-quality resampling filter (a truncated sinc) on all pixels that may contribute
  to the output value. In the current version of PIL, this filter can only be used
  with the resize and thumbnail methods.
Note that the ANTIALIAS filter is the only filter that behaves
properly when downsampling (that is, when converting a large image to a small one). The
BILINEAR and BICUBIC filters use a fixed input environment, and are best used for
scale-preserving geometric transforms and upsamping.
"""

    icon = \
'x\xda\x01\xac\x12S\xed\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\x00\
\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\x08\
\x08|\x08d\x88\x00\x00\x12cIDATh\x81\xdd\x98{\x8c]\xd7u\xde\xbf\xb5\xf7>\x8f\
\xfb\x9c;3\x1cr\x86\xa4(\x8a4i\xbd\x1f\xb6#\xc3v"\xab\xb6\xeb\xd85P5AS4\xa8\
\x93&m\x95\xa4\xb5k\x04h\xd0\x06m\xda\x04(\xdc"\x89\x8a$H\x11\xd8R] n\x80\
\xa8-P\xa7u\xe2\xc8i,K\x8amI\xa6\x9e\xa4H\x8a\xe4\xf0=\x8f;3\xf7}\xcf{\xef\
\xbdV\xff\x18Z\xb5l\x8b&\xcd\x04\x05\xfa\x01\x07\xb8\xb8\xd88\xeb\xfb\x9d\
\xbd\xcf\xfa\xf6>\x84\x1f@\xff\xe1\x99\x87PT9nY:\xa4:\x8dyet`\xb4\xa2\x80\
\x88\x02\x00!\x81\x8c\x10+\x16\x0b\x11\x07\x06C\xc4{/\xce\x89\xb8\x8a\xc06$m\
k*\xf4\x8b\xe1\x92?8\xfba\x81\xed\x83j\xef\xbfn/t-\x83\xfe\xc5\x93w#\xd0!\
\xee^|w\xd0\x08g\xe20\x88\x9b\xc6\x04\x1dE4GD\xf3D\x98\x13pG\xc0m\x814\x05\
\x1c{q\xdaIE,N\xbc\xb0\xb08\xe7\xb8*,\x17S\xc7\xc5\x98\xc5\x8f\x00\x1ah\xe8~\
\xa0\xcc0R\xe1\xa8\xaej\xe9\x9ep\xae<\xd0\xf9y\x17\x99\xdd \xfa\xfe\xf6\xder\
\xc4\xef\x1f\xffG(\xab\x9c\xe6\xdb\x0bq#l\xcdF&^2:\xd8\xa7\x14\xed\'\x85} \
\xde\xc3\xf0\xbb\x18~\xce\x8bmy\xb8\xd8\xb1\x8d\x9cX\xe3\xc4*\xeb-9\xa9\xc8\
\xb1\x83\x17\x16\xcfN\x1cW>\xf7\xa9\xcb\xdc\xa4\xb4\xecr\x11=!\x98\xbeF\xd05\
*\\\t\x11\\\x8aux\xa1\xae\xe3\xcbmS\xef\xde\x14\xcf\x8c\xdf\xd5\xd9_n\x95c\
\xecl\xff\x9bk\x03x\xfc\xd4/ 4QX\x0f[\xf3\xa1\x89n\t\x83\xe8v\xa3\xcc\x1d\
\xa4p\x98\x94\xdf\xeb\xc9.xTM\'el\xb94\x15\x97d\xa5\x82\xe5\n\x96-\x9c\xb7\
\xb0\xec\xe0\xae\\\x9e=\x9c0<3\n\x9faXmbd\xfb(\x9c\x83g\r\x96\x80!\x81\xd5\
\x88\nC\xd1$\xd6\xf1VM\xd7.5u\xfdT+\xa8\x1d\x9f\tj\':A|y\xa7i\x0e\xda*v\xef\
\xde\xfb;o\x06\x18\xff3\xc1\x97~\xf1a\xb4d\x81\x0c\x99Fh\x82\x9b\x03\x1d\xde\
\x13\x04\xe1\xfdF\xab\xbbH\xf1~Vv\x87CQsR\x18+\x05Y)\xb7\xcd\xbeq\xf9m\xa3\
\xcc\xe2\xff\xefo\xf8+\xc6=3J_`Tm\xa0W\xae#qS\xca\x1dPz\xa0p@\xe9\x08\x96\
\x15\xbch\x90\x84\x12\xa8\xd8\xd6t=m\x05\xf5\xad\xd9\xb0yv>j\x1c\x9d\r\xeb\
\xdfl\x9b\xf8XC\x85\xab5\x1d\xe4?u\xe8\x0f\xe4\x8f.=\x0cz\xaa\xfbi\xf2\xe0\
\x96\x82z[\xa8\x83\xf7\x85A\xf8\x801\xeaN2~\xc9SQ\xb7\xc8\x8c\x93\x82\xacTpb\
\xf1-\x83\x9e=\x9cgq\xde\x89s,\xce3[\xcf\xec<\xb3\xf7\xcc\x9eY\x1c\xb303\xac\
/id\xb7hh\xd7U\xce\x89\xb2\xe2U%\xa2\n\xcfT8\xa6\xcc\x0beN\x90{\xa0t@\xe5\
\x15X\x0c\x14\x85R\xd3\xb1\xedD\x8ddW\xdcZ]\xac\xb5^\xdd\x157\x9e\x9a\x0b\
\xeb\xcf5Ut\xde\x0f)3\xce\xa2\x1d\x86\xd1G\xe3(\xfcx\x18\xa8{\x95q\xf3^\xa5a\
\x85\x8c\x9cT\xe4a\xb7\x9f\xa4xx\xc7RY\xe1\xb2bW\x96\xbe\xc8K\x9b\x95\x85\
\x9b\x96\x85\x1f\x97\xa5\x9fX\xeb\xa7U\xe5S\xef%c\x96\x8aY\x98\x85%\xb3\x13\
\xdd\xcb{a.E\rZ\xb7\xc8\xe8&\xb4\xcc\xb0\x91\xb6\xd2hE\xc4\r\x81\xc4\x106\
\x0c&\x0b!\xcb\x1e\x85+\xa8\xcfe\xb8\x9a&\xb3\xe7\xcd\xa83\x1f\xd7\x0f\xeek\
\xb6\xdf\xb7\xbf6sd\xb6\x8a?\xcf\x1b\xf2\x94I\x92*h6\xd4\x81(r\xef\x92\xb0\
\xdaU\xaaT<,\x89x\xb00\xace)J\xe7\xd3\xbc\xca\xd3\xd4\x0e\x93\xa4\xdaL\x92r5\
\x9dV\x97\xd3\xa4Z\xc9\x13\xbbVd\xaeW\xe6vT\x16>q\xa5/\xbc\xe3\x8aY\x9c\x08X\
\x84\x91\xf9)\x8d\xcb\xb1\xf1dC\x1dQM\x87\xd4\xd0\x11:*\xa6\x05\x8ah\x89"\
\xbd\x17\x11\xed\x0b\x03\xd9\x03#\xbb\xb4\x96\x8e\x86\xd4\xa1De\xc2TzO\xa5\
\xcb1\xa9\xca\xda\xa4,\xf6O\x91\xaa\xb9I\xf0u\xba\x80o\x98\x13\'\xd7\xc6\x07\
\x0e\xcc\xbf(Ax\xb6Qs;\x94\xb2\xda\xb3\xa0(\x1c\'i\x99O&\xd5\xd6h\x9c\x9f\
\x1f\r\xf3\xd3\xa3Aqj\xd4\xcf\x97\xc7\xfdlu\xdc\xcb\xb7z\x97\xa7\xd3\xb5\xd7\
\x87\xc5\xe6\x99\x89\x15\x07\x06 \xdf\xb7\xef}\xaby\x04P\xcdC0\x9d\xb7\xe9\
\xa8\xb6\xa0[\xa6\xad\x16LC\xed\xa5\x86:\x18\xd4\xd5m\xcd\x08\xb7\x06\x01\
\xf6\x87F\x162\xe2ZNL\x02\xc0\x16\xce\xf6F\xd3\xe5\xe1Ew|\xf2\x95<%\x91\x97\
\xf1{O=\xb6\xb4\xfb\xa6\xf6\xa7\xdas\xf4\xb0\xe8\xa23\x9e\xe6\xa3~?\xbb0\xe8\
\xa7\xc7z\x9b\xe9\xcb[\xdd\xe4\xf8\xd6Zramy\xd8?\xfe\xd4Jj\x07\xde}\xa7\xd9G\
^\xfa\x08\xac\x94\x981;H\xa1R\x9e2"(RP\x00\xb18\xe7d\x9c\x0cx4\xedK`B\xfc\
\xbb\x0f/\xbf\x19I\x01\xcd\xfd0\xf3\xf7\x05\xf5p\xa7\x99\xd7\x1d\xb5_\x9a\
\xea._W\xef\xb0\xb1\xdc]\x86\xd8/\x01\xcd\xc4\xa5\xda\xd4k\xee3\xc5+\xe5\xa3\
k_\xa8\xd6\xe9\xb3_\xff$6V\x13\xb3w\x7f\xfb\x03&\xf6\x9fJ\xcbikss\xfc\xcd\
\xf5\x95\xc9s\xddK\xe3\x93\x97N\x0f\xd6\x97_\xd8\x9cN\xceg\xae\xddna2\x99\
\xe2\x95\xc9\xe7\xb0\x95^4\xdeK\xa4@M"\xd5VJ\xb5\x89\xa8\xcdb\x9b\xb9\xef\
\xd7KI"\x01k\x01\x00a\xcf"\xb9\x082\x81L!\x98\x08d",\x13\x11\xa4\x81\n\xaa\
\xc5\xfa\xa2\x7fp\xf1\x11\xec\xdc\xd9\xc1\xe6\xe6\x08\xbb\xde\xa9t\xe3\xedAC\
/\xe8\xddh\xab\xdb\xd1\xd6\xef5u}\xbf*\xa5W]\xaa\xfec\xf7\x8f\xd3\xbf\xe8\
\xbc3\xb0\xf4\xaf\xfe\xf0o\xe3\xd3?\xf9\x04~\xed\x0b\x1f[(\xcb\xf2\xbeAo\xc2\
\x17Oo.\x9f:\xd2\xdd\xbc\xf8B?\x9f\xef\xec\x90\xcd\xcdM<\xb1\xf6+J\x89\x8a4\
\x053Z\xa9E\xa5\xd4~Rt@\x11\xdd\x0c\xc2\x92\x90\xccy\xb1\xad\x8c\x07\xf5\xa9\
\xeb\x86\x85\x9f\x1a+Vmg\x013\x8bX\x80J\x08e$fL\xd0}\x82^W\xd0\x17\x95\xe8s\
\x86\xf4\x85@\x05\x9bu\x1dO\x17k3\x95"\xe2\xf7,\xfc\x16L\r\xd8\xfd\xa1 \x8an\
\x0ev\x86m}X*Q\xd9\x8a}\xe5\xc2\xe3\xe5V\xf3 @\x7f\xed\x9f\x1f\xc0\x9e\x99\
\xbd\xf0\xda\xd2\xd6\x85i\xb0vf(+\xaf\x0f\xec\xbf\xfc\xec\xdf\xc4-\xb7wP\xd7\
\x9d\xb8f\xea\xf3\x81\x0e\xf7k\xad\xef\xd0J\xdd\r\x85[A~/\x93\x9d\xb5(k\x96\
\x8b\xb0\xe4B\xa7~H\xfd\xea\x02\x8d\xed\x06\x95\xbe\x80\xf5\xeeJ\xa8mg\x013\
\x84\x85D\xc40\xc4xBT\x19\xaae!\xc5\xfdP\xc5\x97B\x15\x9d\x8cUt\xb4\xae\xa2\
\xd7\xdb\xa6vaO<;|p\xe9\xd6\xea\xbf<\xf95\xfc\xdb\xdf\xfe\x02\xd8\x90qVT\xd1\
\x13\xdb\xb9U\xc9\xf2\xe7\xedw\'\xf1\xa3_\xf89\x1c|\xef.UU\xdc\x88L\xbc/4\
\xd1=F\xeb\xfb\xb5\xa6{\xa1\xf8\x80\xa7j\xceJ\x1eV\x92\xe9\x923\xaa\xb8\x84\
\xe5\x8aJ\xce0\xb2\xeb\x18U\xab\xc8}\n\xcb\x1e\xde{X\xf6p\xde\xc3z\x86c\x86e\
\x86\xf5\x82\xca\x13*\xaf\xc4z-"\x01\x94\x8a|\xac\xeay]7z-\xd38\xdb2\xf5\x97\
\xdaA\xfd\xf9\xd9\xb0\xf1\xdaRcf\xf5\xee]\x07s\xda\xaa\xc9\xe1=\xbf\xf4\xd6[\
\x89g\xd6\x7f\x03\x004\x91\x1c2F\x7f\xc0\x18\xf3#J\xe3^\xd1v\x8f\xa7\xbcQIF\
\x15\xe7\xa8\xb8\x80\x13KW\x96\x07\x1c;Lm\x0f\x83r\x05\xa9\x9d\xa0rN\x9c\xdf\
N\xe6\xed\xa7\xef\xc9\xb1\x87\x17&\x87m\x88\xc2\x0b\n\'\xc8\x9d w@\xe1 \x96\
\x15\x80\x00\x91\xaeq\xcb4\xa7\xb3a\xeb\xe2\x8e\xb8\xf5\xd2\xce\xb0\xf5t\\\
\x04O\xf7\xce\xa5\x17\x8b\xc4\xcac\xbf\xf0\xdc\x1b\x9e\xcd\xb7\x03\x14\x85\
\x05\x80@i\xf5\xce \x0c\x1e6\x91\xbf\xd5\xa94\xae$C%9\x9cT\xf0\xd8\xce\x07\
\xef!\xd6\x92\x14\xa5\xb8$\xcb\xca\x8d\xa4\x9f\x8d\xf24++\xce\xac\x95\xc29T\
\x9e\xe1\x99\t\x0c\xadEQ($uh\xd4Xq\x1dJj\x9a$\x0e\xc0\xc6\x81\xc9\x12S!\x8c\
\xdc\x95\x18\x14\x95^\xe3\xb4\x13\xe9\xc9\xcc\xce\xb8\xfd\xb6=h\x1d\xae\x0f\
\xf5`p\xacX\xb3)W\xdf\xee\xf9M\x00\xd3i\x06/b\x95Vk06\xe5\xd8\x1a\x8f\x14n;\
\x8d\xc59\x91\xa2\x14\x9ff6K\x93j\x90\xa6\xb6\x9b$\xe5\xca`<X]\xedou\xa7i:\
\xb0\x15\x8f]%\xa9\xb3\\1\x8b\xdfn\x91\xa2T@\x912h\xa8\x80fT\xa8\xe6)\xc4"E\
\xd8#\x01\xf6\xea@\x96"#s\xa2|,\xe4\xb5\'\xa1\x8c=\xe5.\'.\x10\x94\x93<3\x97\
0\x1c\xbdh\xd9%\x8c\xb7\x048\xbd\xbc\x86\xca:\xbfs\xb1\xfd\x1a\x05\xcd\xafPL\
\x87uM\x16\xca\xca\xbb$\xa9\xf2\xd1\xb8\xec\x8d\x86\xf9\xd9\xf10?>\x1e\x14\'\
&\xc3\xf2\xc2tPt77\xfb\xc3\x95\xb5^\x9a\x0c\xab\xb2H\xc4\xe5Sa7\x01#\xdf^\
\xa4T\x07\xe2\x1d\xa0ZS\xeb\xda\x8c2\xd1\x8c\n\xa3\x19\xaa\x07-5k\x1aXTu:H\r\
u\x87\x8e\xd5\x9d\xf5\xd8\x1cP\x81\xcc\x93\xe2\x9a\'\xa5\x83)6\xb3n\xf1\xd5\
\xe4xy\xba\xff\xaas\xfeM\xcf\xff{l\xa7\xbfx\xeeWq\xf4\xd5Kz\xef\xbe\xce\x0f\
\xb7f\xf5\xbf\xf6*\xbb\xa7?\x1a_\xde\xd8H^\xdeZO\x8e\xf4\xd6\x92\xa3\xfd\xd5\
de\xd0\xcd\x86\x9b\x17\xa6E\xf7\xf4\xc4\xd9iu\xad\t\xfcf\xcd\x03\x0b\xb7k\
\xd3X\xd2Q\xd8\xa1N\xd0Q\xfbT\x9b\xeeFK\xbd\x9b\x9b\xfa\x1dd\xd4n\x19\xca\
\xf3\xf9\x99\xf2\xd7{_-\x9e\x9b\x9c\x10\xf7\x9d\xb70\xdf\xf9\xc7\xc9\x93\x97\
p\xea\xd9\xdc\xd7\x1a\xeaDo\xe4\xfek\x9a\xa5G\xd6\xd7\x07/\xaf,\x0f^\xbd|b\
\xd0\xed\x9e\x1dg\x1b\xa7\xc6N\x18\xf2O\xbe\xb4\x00\xad\xe6\xe8\xf0\xd2\xdd\
\xa6S\x9f\r\x03\x15\xc4F\x07\x11\x91\x8a\x08\x08\x88@\x02\x80\x04\x02PED\xa5\
\x12]\x06\x14\x963\xba]\xbd\xb3\xf5\x1e\x17R\xe8h\xf1\xaf;2Hw\xdc\xae\xbb\
\xb5\xbd\xeaX\xb4[?\x19\xcd\xd3}&\xc2\xbd~\xcag|\x8fOON\x88\xdb\xf9\xa0\xc1\
\xe6Sof\xf8\xae\x19\xf8\xc4c\x1f\xc2t\xe8\xd0\x9c\tT:\xadf\x07\xdd$|\xfd\xd9\
\xd5\xf1\x99\xafws(\xc8\xd7\xfc/\xe1\xe4\x91M\xd5\xe8P]\x1b\x9e\x0f\x02\xb3\
\xa4\x94^"\xc2\x92\xc0/0\xfc\xac\x17\xd7\xf6\xf0\xb1g\xa7\x9dx\x917\x92X\xa6\
\x80\x1a(\xa8M\x05\xbdf(\\\x8f(\xea\xb6tc\xb8\xd3\xccf\x05W\xf2\xc0\xdb\x7f\
\x03v\n\xec\xfb[A\x14\xcf\xa9Y\x97\xc0%\x97\xfc\xf0\xef=\xbb\xd3\x7fv\xf7:\
\xb2u\xb9:\xc0\xf7\xd2\x9e\xa3\xc0\'\xe2\x1f\xc3\x1d\xc1\xc1\xb8\x166w\x85\
\xda\x1c\xf2\x94\xdcY\xd1\xe46O\xd5\x01\'\xe5\xaeJ\xf2N\xc5y\xa3\xe4<,87\x95\
\xaf\x94\xe5\n\xdb\xed\xd4\xc3\x0b\xb3\x089\x88*\x01\x93(\x04C\x8d\xa8k(:\
\x1b\xab\xf8DM\xd5^k\xa8\xda\xd9\x8ejn\xd5)\xae>\xb8\xffw\xaf\xc5\xda\xd5\
\x01\xbe1\xfcM|\xf1\xcb_\xc2\x07~\xe4\xc1\x9a\xd1\xea\xa0\xd1\xd1\x0f\x05\
\xc6\xbc\x07T\xdd\x95\xa2{\xd3\x84\xd7;\x19\x8f\xe2\xdc%*\xe7\x84J\x97\xa3\
\xf0\x05*_\xa1\xe2+GKw%\xcc\xd8\x83\x19p\x020+\x08\x1b\x11\x84^!\xceC\xaa\r\
\xea\xbay\xa1\x194\x8e\xb6t\xe3\xd9\x96n\xbc\x10\xf9\xda\xc5z3\xa8\x9e~b\x19\
\x8f\xfe\xfc7\xde\xd2\xe3w\xbd\x03\xdf\xae4\xb1x\xff\xfd\x1fP\x9a\xc2;\xe3 \
\xf8\xb9 \xd4\x0f\xb2*\x16\xa7\xdc\xad\xf5\xdci=\xe1\r\x94.G\xc5\x15*o\xe1\
\xf8\xca\xd6A<\x98\x19\xe2\x05,\x02\x88@D\xe0e;\x8d+/(\x1d\xa8\xf40\xd6\xab\
\x96\xe7\xa0\xa9(\xdcS\xd7\xcd{\xe7\xc3\xce\x07g\xa5\xfde52\xff\xe9\xd1\x8f\
\x9d;\xf1c\x8f\xec\xb9\xea\x0c\\\x15`\xb3?\xc1\x85\xb5\xae\xdcq`\xbf\xa1\x19\
\xbe\x89C\xbb/\x97^8\xe2\x8b\x98Jw;\x91\xd9\xc1Y\x11g\x89K\xab|Yig+X\xeb\xc8\
;\xaf\xbcg\r\x86\xd7\x0c6Ll\x88\xd8h\xf2FCH\xc1\x93\xdf\x0e/*}i\xfaR\xcc\x0c\
\xb9\x08k\xa3\xd1\xcd|\x99\xa2\x04\xebx\xf9\xcf\xec\x0f\x0ePU\x0e\xf3\xcd\
\xa6\xacu\xb7\x96\x1d\x1a\xcf5\x8d\xbf\xcf\xc7\xe3\x85\xc2O\xa8,\x9d\xa4\x99\
\xd8$\xe1i\x92\xb8a\x96\xba^\x96\xba\xad"\xf7\xfd\xaa\xe4\x89\xb3\x92z\xcf\
\xa5@\x00%!ij\xaa@\xcf\xa8P\xedP\xa1^\x90\x00;\xc8H\'\xd0\xbe\xe9\xc0\x81#O\
\x9e!\xf9\xc4\xf6&\x97\xed\x91\xf17\xed\xe5\x1f?7\x87\xff\xf9\xf6\xfe\x0f\
\x0e\xf0\xb3\xf7?\x82_}\xfc\xef`:*\xfa:^\xfc\xaa5\xf2\x80+G\xef\xddH&Eo\x98\
\xad\x0f{\xe5\x99\xe1\xa0:9\x19\xda\xe5dd/gc\xbfYL\xfd\xa4\xca$\xb7\x85T\xde\
\x8a\'\x02LL\xca\xd4\x11\x86\r\xaa\x05u5\x136\xf5\xce\xb0I\xfbt\x83\x0eQ]\
\xdd\xa6c9\\\x0b\xb1\xc4\xact5\xc5\xb1r\xdd\xfe\xc5\xeaW\xcaQ\xff\x83\x15\
\xbc\xbdz\xc4\\S\x17\xfa\x95?\xfa(Ha.j\xf2\xcfd<\xf8p\xb7\xbfync=}\xa1\xbfZ\
\x1e\xef\xaf\x14\x977\xce\xd8\xf1\xe4U\x94\xb0\xf8\xae\xa0\xf9^\xd2u\xe8\x85\
\x07t4ss\xd0\xa9\xedP7\x85\x1dug\xd0\xd6\xf7\x13\xd1\xcdY\xd7\xff\xc9\xe6\
\x91\xe2\x0fW\xbeh{\xd7r\xafk\x02x\xe8\x91\xbd0\xda\x901f\x7f\x92\xe4{\xd7.\
\x8e\xbb\x97_\xcb\xd6{_\xe3\x0c\x00\xbf\xeb]{q\xe4\xc8\xa3xi\xfa\xa78;\xe9+_\
\xb1\xf2\xcc\x1a\x80"\x10\x00bE\xca\x87A\xc0\xb7\xb4\xf7\xf0}3\x9f\xc6\xfa\
\xfa\xef\xe1\xf6\x87>\x89j,\xb4p_X\x9f\xd9\x17,iC\xbb\xb3\r\xbf\xd2\xfdZy\
\xd1&\xe2\xb3\xb5\xef\x1f\xf0\xd7\x04\x00\x00\xf8a@3(\xcc\xa0\xf2c\xe0\x1f\
\xff\xa7\xfb\xe5S\xbfu+.\x9d\xadiMaM+3\xabH\xedP\x8av\x02\x98\x17p\x9b\t1\
\x84\xc5\x8b\xcfE0\x81H\x0f\xa0\r\x05\xdd7\x14\x8c\x1bz\xa6\xf8\xe8\xde\xdf\
\xf1\xbf\xfc\xcb\xff\x18\xbf\xf9\xdb\x9f\xc1\xec]\xd0.\x83d\xab`;\xbe6[\xd7\
\x0e\x00@D\xf0\x9f/\xbf\x17\xfbZ\xf7\xa3?\xd9\x8c\x95\xe8\xc5@\x87\x87\xb5Vw\
\t\xc9\x1d\x0c\xb7\xdf\xa1Z\xb0R6+.\xa3J*m}\x05\xcf\xde\xb1HIBS\x88\xdeP\xd0\
\xe7\x14\x82\xe3\x86\xc2\xd7b\x1d\x9fi\x86\xad\xadf\xdc,s\x9b\xe0\xa1]\x9f\
\xbb\xa6\x8f\xba\xdf\xd2U_\xe2\xef\xd4\x7f;\xf7\x0f\xf0\x0f\x0f^\xc6\x7f?\
\x7f\xebl\x1c4\xdf\x1f\x18\xf5\x11hy\x87C\xb1\xaf\x90I;\xf5\xe3 qc\x95\xb9\
\x04\x85\xcf\xa9\xe4\n\xd6\xb9+I,\x02Q\x021\xb7k\x84\xef\x0b\xa8>\x8aT\xfdBL\
\xcdo\x9aQ\xed\x89\xa2\x87o<\xfc\xa1=\xd3_\xfc\x83\x07\xaf\xc7\xd2\xf5\x01L\
\xa7\x05\x80\x15d\xa9\x9b\xad7\xd4G(\xb4\x7f\xb7P\xfd\x99\x89\xdbB\xe2F\xc8]\
\x8a\xcaW\xb0\xec\xde8J:\xd9Nb\xeb=9f8\xcf\xb0LF\xc4\xd4\r5\x96b7s\x88\x86u\
\x97^\xa4\x13\xbb?t\xd7\xf4\xe4+_\xff\xab\x03\xe8\xf7\xa7\xf8\xdd\xe7\x7f\
\x02\xdd\xeed4\xb3#8\x17\xaa4)\xa3\xeeL\xc6\x13X\xb6\xc2"`\xa7\x88\x9d\x16oI\
\xbcW\xe2E\x81Y\x11\x91"\x05O\x8a<\x00F\xc5\x16\xb9Mi8\xf0Iza\xbc28*\xc9O\
\x9c\xf8},\xbfxM\x8d\xec\r\xe9\xeb\x19\xfc\xe7\x9f?\x8d[\x1eh\xe1\xcb\x7f\
\xf2z\xb1\xef`\xdb\x99\xc8\x1f\x16S\xdd\\9\xab\xd2\xc4W\x93\xa1\x8c\xc7}^\
\x1bn\xf1\xf9\xf1\x16\x9f\x9a\x0e\xe4t2\xc4\xd9l\x8c\xd5*Q\x03[\xe8B\x9c\x16\
@)"\xa3\xb90E\xd2\xa5g6N\x94\x8f\x9fx,=\xdf=\x9a\xcb\xda3\xe5u\x01\\\xd7\x0c\
\x00\xc0\xf1\x93\x97\xf1\x93\x7f\xff>\xe9mf\xa7\x82\x9az\xda\xa6t\xa8\x97X\
\xf4z\xc5\x89\xe9\xc0\x1e\xcf\xc6\xeel\x91\xf2FU\xc8\x84\xbdT \x88\n\x10\x99\
\x98\xdaA\x9d\x16\xc3\xa6z[\xd0Rw\x85uu\x9b\xafTQ\xf5\xfd3\x93\xf3~\xf9G\xff\
\xb4\xc5\x7f\xf6\xd0\xf4z\xed\\?\xc0\xd3\xff~\r\xb7\xbd}\td\xc3\xc9\xa8H\xff\
wR\xa6eok:\xda\xbc\x94\x1d\x9bn\xb8n\xb2\xe5\xa7I\xcfW\xd9*<\x8f P\x80\xda\
\x05\x9a;\xa4Ts\x87\t\xebs\xaaY\xdbA\xbbk\xb3\xb8\x1b"3\xc9\x9a\x7ff\xed\xcf\
\xdd\xa4\xd8H!\xd5\xf7\xaf\x7f\xc3\x00\x000M3\x9c?{\x96\xe3Zpr\xbae/n\x9d-\
\xdc\xe6\xe9\xb2\xc8V\xaf|\xe2\x050\xff3@l\x80\xdb;\n5\r\xf9_\xbf\xce\xdcC\
\xe5f\xefSY}\x87\xea\xd5\xe6\xfci\xd2\x14\xe5[\x9c\xa3\x82\x1f\x1e\xe3\xab\
\xd6|+]W\x0e\xbc\x95>\xf5\xd5\x9dx\xf2\xf4&>\xfeC\xf7\x9b\xc5\xe6\x9eV\xcd4:\
F\x07-"\n\xb7\x93\x18\x05@S#\xc1h\x0e\xbb\x92\xf7=\xfek~\xf2S_F{\xf7\x8f\xde\
p\xed\x1b\x06\xf8\xe3\xb5O\xc0si<\xfb=D\xb8G\x1b}\x97@\x0eX)v\x94\\D\x96+x\
\xf6\x19Cz$j9@xTlp\xb4\x98R\xf7\xb1On\xf0\x8b\xcf>uC\xf5\x7f\xa0%\xf4\xed\
\x1a\x0e\x12\x98P5\x82@\x7f,\x8c\xe9\xa7+\x9d\xdc2q\xfd\xe6\xd4\x8e\x82\xcc\
\xa5\xaa\xf2%<{f!k\x10LT\xd58\x95\xf7\xcc\xe7\xb6\xce\xda\xffQ\xd6\xfb\xe9\
\x8d\xd6\xbfa\x80\xb3\xcb\x9b\x98[h\xb8\xb8\xa1%\x90b)\tVw\x0e\xab\rd>\xdb\
\xfe\xb8\xeb\x1d,{-,\x01|T\xcb\xb7\xe2\xc9\xf0\x9c6\x1b\xafx\xbc\xf6\x95\xeb\
\xef:\x7f\xe9\x00V\x158{&\xcfv\xec\r\x9e\xaf\xc3\xbd\xe4\x9b\xc5\xa2E\x15n\
\x1f)\x01a\x02\x84 \x02\x14)\xe7\xbdn\xfe\xe2\xca\xeb\xee\x95\xd3O\x94\x85\
\xc8\x9d :zC\xf5\xd5\x8d\x02\xf4\xfb\x19\xfe\xc6\xc7o\x92\xe9\xd8.\x97)\x9et\
i\xbcVM\xa3"\xe9\xeb\xfetK\xaf\xa6=\xb3V\x8d\x83\x81O\xa3\xb2\x1c\xe9\x95dS\
\x9e\xee/\xdb\xf3\xf9%\xf6\x9d{\x8e\xddh\xf9\x1b\x9f\x81\xcf\xfc\xec\xf3\xa8\
54\xdc4\x9aJP<S\r\xca\x83\x83\x8cw\rF\xee\\Ur\x1f\x04D5,\x98\x98n)&\xbc2]\
\xe7gW\x9ew\x93\xc3?\x1d\xe1\xf4\xe7\xaf/u\xffJ\x00\x00\xe0\xf2\xda&\xb6\xc6\
+2?Y<\x9dg\xe5g\x86\xfd\xcct/e\xbdt\xc4\x99\x0e\x81\x99%Sk\xcc\xe9\x05_\nO/\
\xfb\x8b\xe8\x83\xb7^\xb8\xfaa\xfd\xff\xa5\xae\xd6\x9a\xffRr\xe7\xff+\xfd\
\x1f!\xff\xfe\x92\xfa\x99\x9ak\x00\x00\x00\x00IEND\xaeB`\x82\x9dK\x87\x1a'
