#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Phatch recommends SPE (http://pythonide.stani.be) for editing python files.

# Always import this:
from core import models
from core.translation import _t

# Declare constants here

CROSS                   = _t('Cross')
ROUNDED                 = _t('Rounded')
SQUARE                  = _t('Square')

CORNERS                 = [ROUNDED,SQUARE,CROSS]
CORNER_ID               = 'rounded_corner_r%d_f%d'
CROSS_POS               = (CROSS,CROSS,CROSS,CROSS)
ROUNDED_POS             = (ROUNDED,ROUNDED,ROUNDED,ROUNDED)
ROUNDED_RECTANGLE_ID    = 'rounded_rectangle_r%d_f%d_s%s_p%s'

def round_image(image,cache={},radius=100,fill=255,pos=ROUNDED_POS,
        back_colour='#FFFFFF'):
    if image.mode != 'RGBA':
        image = image.convert('RGBA')
    mask    = create_rounded_rectangle(image.size,cache,radius,fill,pos)
    image.paste(Image.new('RGB',image.size,back_colour),(0,0),
        ImageChops.invert(mask))
    image.putalpha(mask)
    return image

def create_corner(radius=100,fill=255,factor=2):
    corner  = Image.new('L',(factor*radius,factor*radius),0)
    draw    = ImageDraw.Draw(corner)
    draw.pieslice((0,0,2*factor*radius,2*factor*radius),180,270,fill=fill)
    corner  = corner.resize((radius,radius),Image.ANTIALIAS)
    return corner

def create_rounded_rectangle(size=(600,400),cache={},radius=100,fill=255,
        pos=ROUNDED_POS):
    #rounded_rectangle
    im_x, im_y  = size
    rounded_rectangle_id    = ROUNDED_RECTANGLE_ID%(radius,fill,size,pos)
    if cache.has_key(rounded_rectangle_id):
        return cache[rounded_rectangle_id]
    else:
        #cross
        cross_id    = ROUNDED_RECTANGLE_ID%(radius,fill,size,CROSS_POS)
        if cache.has_key(cross_id):
            cross   = cache[cross_id]
        else:
            cross   = cache[cross_id]   = Image.new('L',size,0)
            draw    = ImageDraw.Draw(cross)
            draw.rectangle((radius,0,im_x-radius,im_y),fill=fill)
            draw.rectangle((0,radius,im_x,im_y-radius),fill=fill)
        if pos==CROSS_POS: 
            return cross
        #corner
        corner_id   = CORNER_ID%(radius,fill)
        if cache.has_key(corner_id):
            corner  = cache[corner_id]
        else:
            corner  = cache[corner_id] = create_corner(radius,fill)
        #rounded rectangle
        rectangle   = Image.new('L',(radius,radius),255)
        rounded_rectangle   = cross.copy()
        for index, angle in enumerate(pos):
            if angle == CROSS:
                break
            if angle == ROUNDED:
                element = corner
            else:
                element = rectangle
            if index%2:
                x       = im_x-radius
                element = element.transpose(Image.FLIP_LEFT_RIGHT)
            else:
                x       = 0
            if index < 2:
                y       = 0
            else:
                y       = im_y-radius
                element = element.transpose(Image.FLIP_TOP_BOTTOM)
            rounded_rectangle.paste(element,(x,y))
        cache[rounded_rectangle_id] = rounded_rectangle
        return rounded_rectangle

class Action(models.Action):
    
    label       = _t('Round')
    author      = 'Stani'
    email       = 'spe.stani.be@gmail.com'
    version     = '0.1'
    tags        = [_t('filter')]
    __doc__     = _t('Rounded or crossed corners')
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Radius')]                = models.PixelField('5%')
        fields[_t('Top Left Corner')]       = models.ChoiceField(CORNERS[0],
                                                CORNERS)
        fields[_t('Top Right Corner')]      = models.ChoiceField(CORNERS[0],
                                                CORNERS)
        fields[_t('Bottom Left Corner')]    = models.ChoiceField(CORNERS[0],
                                                CORNERS)
        fields[_t('Bottom Right Corner')]   = models.ChoiceField(CORNERS[0],
                                                CORNERS)
        fields[_t('Background Colour')]     = models.ColourField('#FFFFFF')
        fields[_t('Opacity')]               = models.SliderField(100,1,100)
        
        super(Action,self).__init__(fields)
        
    def import_modules(self):
        global Image, ImageChops, ImageDraw
        import Image, ImageChops, ImageDraw
        
    def apply(self,photo,setting,cache):
        #get info (always get this)
        info            = photo.get_info()
        width, height   = info['new_size']
        dpi             = info['new_dpi']
        
        average         = (width+height/2)
        radius          = self.get_field_size('Radius',info,average,dpi)
        fill            = int((self.get_field_value('Opacity',info)/100.0)*255)
        pos             = [
                            self.get_field_value('Top Left Corner',info),
                            self.get_field_value('Top Right Corner',info),
                            self.get_field_value('Bottom Left Corner',info),
                            self.get_field_value('Bottom Right Corner',info),
        ]
        
        parameters  = {
            'cache'         : cache,
            'radius'        : radius,
            'fill'          : fill,
            'pos'           : pos,
            'back_colour'   : self.get_field_value('Background Colour',info)
        }
        
        #manipulate layer (always return)
        return self.apply_to_current_layer_image(photo,round_image,
                    **parameters) 
                    

    icon = \
'x\xda\x01\x80\x11\x7f\xee\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\
\x00\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\
\x08\x08|\x08d\x88\x00\x00\x117IDATh\x81\xb5\x9ay\x90eU}\xc7?\xe7\xdc\xe5\
\xed\xaf_\xef\xd3\xdb\xec\xc3l\x8c0\xa3\x80\xc2 R\n\xe2niR\t\xa64\xa8\x89\
\x1d-\xd4,T\x91X&\x06cJ,\xcb2\xc1\xa0iKSQ+\x80D\x91\x10\x8d\x8a$q\x06%\x02\
\x02\xce\xce\xec\xcdL\xef\xfb\xdb\xdf\xbb\xf7\x9es\xf2\xc7\xb9\xdd\xd33\xf4\
\x18\x16\xf3\xab:u\xdf}\xdd\xf7\x9e\xdf\xf7\xf7\xfb\xfe\x96s\xce\x13\xc6\x18\
^\xae\x88\xefM\xb4\xd0\xd0\xebES\xed\x10Fo\x05Va\xc8\x0bc\xca\x02=%\xb5:\xea\
\x89\xf0`W0~\xfa\x86}\xf7\xcf\x01jhh\xe8\xe5O\x0c\x88\x97\n@|g\xbcG6\xd55&2\
\xd7\xa3x\x15\x82\xb5\xc6\x97\xedYW8\t)H\x08Ad\x0cMm\xa8(\xa3L\xa0\xe6e\xa4\
\xcf::\xf8\x95\'\xc2\x9f\xb6\x9a\xd9\x9f\xbf\xe9\xa9\x7f\x19\x01"@\xbfT@/\
\x1a\x80\xb8ol\x1d\x91\xf9\x88P\xfa\xedF\xcaK6\xb4x\xbc\xa6\xc3cG\xab\xc7\
\xf6\x82\xc7\xaa\xa4$\xe1\x08|)P\xc6\xd0P\x86rh8V\x8e88\x1f\xf2\xe4\\\xc8\
\x13\xb3\x01\xa6\x16\x9e\xf6U\xe3\xe1\xde\xca\xf0W_\xff\xec\x03G\x81\x00\x88\
^,\x90\x17\x0c@|o"OM\xddFdnM\'e\xeb[\xfbR\xdc\xbc&\xc5\xee.\x9f\x8e\x84|\xc1\
\x13\xd6\x95\xe1H)\xe4\xc1\xb3M\xee=S\xe7\xd4L\xa3\x9ci\x96\xbe\xb5s\xe2g_\
\xdc2\xf6\xcc\x04\xd0|1@^\x10\x00q\xff\xf85"P\xff`\xa4\xb8\xfc\xa6\x9e\x14\
\xb7m\xcep]\xa7\x8f\x01*\x91!\xd4\xa0Wx\x8f\x10\xe2\xbc{\t8\x022\xae\xc0w\
\x04g\xab\x8a\x7f<Y\xe3k\xa7\xaa\x14K\xd5\xe3\xfd\xa5S\x9fz\xc3\xe1\xef<\x02\
T\x80`hhH\xbdl\x00\xf2\xde\xd1\x8f\x19e\xfe\xb6%\xe1d?\xb15\xc7{\xfa\xd3\x80\
a6\xd0h\xc0\x01\x96\xbfA/\xbb\x11\x9c\x0f \xfe\x12\x83\xc1\x18\xc8y\x82v_\
\xf2t1\xe2\x13\x07\x8a<5Uot\xd6\'\xee~\xc7\xd3_\xff\x120\x0f\xd4\xf9?\xbc\
\xf1k\x01\x88\xfb\xc6\xee$2\xb7w\xa6\x1c>\xb3-\xcf5m>s\x81\xa6\xae\x0c\x8e\
\x14(m\x08\x94\xc1\x11\xd6\xb2\x88\xf3=\xb1\x82\xfa,wJd@\x18AO\xdaa!\xd2\xfc\
\xd5\xe12\x8fN6ioN\xde\xf7\xee\'\x87\xfe\x1a\x98\xc6z#\xbc\x18\x88\x8b\x02\
\x10\xf7\x8d\xdd\x89\xd2\xb7w\xf8.\x7fyI\x96\xed9\x97\xf9\xc8\x00\x06)\x04\
\xa12\xe80`kg\x82\x9e\x9c\x8f\xef\n^L>\x10\x02*\x81fx>\xe4\xb9\xb2\xa2#\xe5\
\xd2P\xf0\xc9\xa3%\x0e-\x84t\xd5\xc6\xbe\xfb\xce\xa7\xbev\x070\x0e\x94.\x06b\
E\x00\xf2\xde\xd1?6F|\xb1\xe0\n>\xd4\x9f\xe6\x8aV\x8f\x85P\x83\xb0\xb4P\xda\
\xa0\xea\x15\xde\xbc)\xcf\x9a\xf64*\x8aX\xa86\xac\xc5Wf\xcdy\xa2\x8d\xc1\x91\
\x92B.\x03\xc0\x13\xa3\x15\x9e\x9c\x8cX\x95\xf1\x99lh>}\xbc\xc2dS\xb1z\xe1\
\xc4?\xdd\xb4\xff\x9e\xbb\x80\xd1\x8b\x81p\x9f7\xd9}c\xbbA|6\xe5\xc2\x8d\xad\
>=\t\xc9h="\xd2V\x13!\x1c\xe6\xe6\xe7\xb9au\x825\xedi\x1e?2\xccw\x1f;@\xa0\
\x04R\xca%\xeb^\xa8\xf4\xf2{\x83!\x0c#z\ni\x06\xdfr5W\xf6e8:=\xc3\xf1\x05\
\xe8Ny\xbc\xa3#\xc97\'\xeb\x8c\xb4nx\xdf\xd3k_{j\xd7\xf0\xde\x87\x00\r\x94\
\x81\xf0<c\x9f7\xc9\xb7\xc7\xb2\xc2\x98!G\x88\xe4\xee\xacO\xa7\'(\x86\x9aR`\
\xa8D\x86j$\x98\xad5iT\x8al\xef\xceQ\xab7\xf8\xc6\xcf\x8e2\xe3\xb6";\x06\xa0\
\xad\x1f\xd5\xdaO%\xdb\xc7\\\xba\xe7\xbc1\xbbl\xcce\xfa(\xe5\xfa\xf8\xaf\xd3\
E\xee\xfa\xfe/\x00Aw\xcap|\xba\xccpM\xd3\xe5\xc1\xb5y\x0f!\x1d\xf7P\xef\x95\
\x1f-\xa5\xdb\xb6\x00]@jpp\xd0\xb9\xb8\x074\x7fn\x8c\xd8\xb6=\xe3\xd0\xef\
\x0bj\xdaPWz\x89\xdb\xc2\x81r\xad\x81\x174\xf0\x1c\x98(\xd5i\xe6\xbbi\xeb\
\x19\xc0\xf3<j\x81\xc2E\xb3\xab\xd5\xa3;\xe3!8?C-\x17\x83\xa1\xb1\xbd\x0f\
\xb7\xb1\x00\xc0%\x1d\x19v\xd7`*\xd4\x1c\xaf(\xfa\\\xc1\xf6\xb4\xc3~\x95\xee\
~l\xc3M\x7fp\xd3\x81{>\x8f\xcdJ\xe1\xe0\xe0`c\x91JK\x00\xc4\xb7\xc7\xd6`\xb8\
5\xef\n\xb6&%\rm\x90FP\tM<!\x08m(7#\x92A\x80\xd1\x1a#\x1dt"K\x1d\x8f\xd9JH\
\xa7\x1b\xf0\xfe\x1dml\xedH\\D\xed\x0b%\r\xf4\x00\xb0\xb6=\xc3\xfb\xdbmL\xdc\
s\xa4\xc8\xfd\'\xeblI\xf9\x0c7\x15c\x85u\xd7\x8d\xb6\xae{\xa4o\xfet\x19\xa8a\
\xdb\x8f\xf0|\x0fh>\x86\xa0es\xda\xc1\x17P\xd3 \x8d\xa6\x12I\xe4"\x00a\xa8F\
\n\x11\x99\xd8\xb2\x82\x9a24\x1a!n\xbd\xc4\x87\xaf\xedf][\x82\xbdG\x9e\xe3\
\xf1\x93\x13hc3\xd6J"\x10\x84aDRUy\xcb\xd6U\x1c\x9f*s\xaab\xf8\xad\xdd\x97\
\xf1\x9e\xad-LW\x9b\xec\x19\x0f\xd9\x96r\xf9E\xa8\x13\xfb\x07\xae~G\xdf\xfc\
\xe9\x13\xc0\x02P\x1f\x1c\x1c\x8c\x86\x86\x86\x8c\x1b[\xbfK\x18\xf3\xae\xac+\
\x19p\xadR\xca\xd8\xa8\xa9\x1b\x8d\x8cs\xbc\x14\x10iCh, m\x0c\x81\xd6\xd4J\
\x15n\xe8\xd2\xackK\xf3\x83g\x8es\xc7\xbf=\x89\xf4\xd3H\xf7\x1c]\xcd\x92\xe2\
\xf1UJ\x1a\xf5:\x85\xc6\x0c7\xac\xcdrbt\x92\xcf\xffl\x98_\x9e\x99\xe7K\xb7\
\xbc\x81\xdf\xddR`\xcf\xc8\x18=\xc9,9O2\x9d\xeb\xbdt.\xdb\xb5\xa9\xad25\x13\
\x83h\x02\xa1\x0b \xb5~\xad\x16rm\x9f\'p\x80\xba\xb6\xca\x02(\x03\x8e\x81\
\xa6\x86z\xa4X\xa8D\xb4V\x02\xb41\x84\xda0V\x0e)5k\xb4\xf4d\x01\xf8\xee\xfe\
\t\x8a\xadk\xe9Y\xb5\n\x84\xc4\x00\xbe\x84\x8cw\xce\x93K\x00ju:\x9a\x93\\\
\xfa\x8a\xcb9\xe3\x8e\xd1;\x9dg\xdfB\x99\xc3c\xb3\xbczS\x1fI\xdd\xa0\x1a\xa5\
\x18\xf0\x1d\x0eG\xe9\x96\xe1\x8e-\xbb\xda*S\'\x81I\xa0288\x18\xb9\xe2?\xc7\
\x10F\xbc^H\xe8r%MeP,k\t\x84`\xb6\xa9\xc8\xb8\x82M\x05\x1f\x95J\x91ki\xc5s\
\x1c\xd2\t\xc1e=\x19\xea\x8d\x06}\x85\x14\x00\x9b\xd7\xf4"\xfb\xb2d3)0\x96j\
\xb3u\xc5\x99\x8a"\xe1\xda6\xdb\xc4\x00\x9a\x1e\x18\xd2\xe0\xa7\x90\xc9,"\
\xdf\x81\x93\xc8\x10\xc6~RA\x83\x86\x8c\xe8\xf6\x12<\x0bL\xe6\x07\xb6\x00\
\x1d@+0\x074\\\xe6\xc8\x02;3B\x90\x16\x86 \xa6\x8e\x00\x84\x90,\xd4\x1a\\\
\xdd\xe5\xf1\x91\xcb\xdb\xd8\x90\xf7\xe2g\x07\x00h\xc9\xc0W\xdf\xb4\xee<n\
\xdf\xfe\xba\r\xcf\xe3{M\xc1\x8f\x9e\xabr\xf7\xc1\x12\xf3Z\x90r\x00\xe3P\xd7\
\x9a@\xd9~Mi\x15G\xa5\x8f\x10\xb6\xc3RZ\x13hCZB\xda\x81R\xaamU\xcd\xcfv\xa6\
\x83J\x01H\x01%\x17\xcdF`]>\xa6kdl\x8a\x93B2_\xa9qE\x8b\xe6\xce\xab{\xf1\\\
\xc1\x9e#g8=["\xa9\x03\xae\xea/P\x8f\x0c\xcfL\xd7\x89T\xc4%->\xeb\xda\xb3<9Z\
\xa2\xa8\x1d\\G\xa2\xb5\xc1\x91p\xd5\xdan\xde\xb5\xbe\x95v\x1f\xfe\xec\x7f\
\xe6\xd0\xc6\xc3s\xa1\xa9\r\xa1Y\x8c\n\x83\xd6\xda6zK\xf45D\xc6 0\xb4H\xc1\
\xb4\x97\xc9O\xe5\xfbV\xaf\x9d9z\x0c\xc8\x00\xae+\xb4\xd9a\x84hOI\x1b\x94\
\x91\xb1\xa1V\x8f"t\xad\xc8\x87\xae]\x8d\xe7\n\xfe\xf4\xbeG\xb9\xff\x993\x84\
J\x93\xadO\xf1\xe5\xb7]\xcaH9\xe0\xb6G\xc7\xa9\xd6\x1b|\xf4\x15\x05n\xb9b=w\
\xfe\xe0\x08\x07\x9bi2\xc9\x04hM\x14\x06\x14\\\xc37\xde\x7f\x03\xd7m\xec\xe5\
\xda\xae"\x0f\x8e\x85tel|\x9cS\x1f"\xad1\xda\xb0\xd8\xde(\xa5\t\xb5FkCJB\xe8\
\xf8\xc9\x99\\O\xdf\xda\x99\xa3yl\x0e\xf6]`\x1b\x02\xc7\x03T<\x84\x14\xd4\
\xaaM\xbaE\x9d\xf5\x85\x14\xc7&\x16\xf8\xf7Sez\xb7\xbc\x02\xa5\x15-\x95\t\
\xb6]\xba\x83|9\xa4o\xb2\x8db\xad\xce\xfa-\xadl\xdd\xb1\x89\x81\xe3\x92\xc9F\
\x86|*\t\x18D\x14r\xf6\xb9\xd3\xdc\xbd\xe7\x00\xaf\xd9\xd8\xcb\xce6\x87\x87\
\xce\xd6qd\n\x84X\xeaN\x8d\xb1s\xdbv\xd1~\xa9\x01\xa5\x14\n\xf0\x00#\x85(\
\xa6;\xbab\xebg\x81\x84+0\xdd\x06\xfb"e\x0c\xda\x800\xa0\xa2\x007j"\x8cf\xb2\
\xa6H\xad\x1a\xc0\xef\xec\xa7R\xad i\x90m\xed\xa0\x90\x08I\xe6j\x04N\x83TK\
\x017S \xdd\xd2\x8e\xe7e\xf0\xd36\x88\x85\x14tiA\xdd\xcc\x01\x90D\xe1E\x01\
\x18\x8366a\x9c\xa3\xd09\xe5-\x00\xab\x8f2\xe7\x806\xddd6\xe6\x7f\x1aH\xb8@\
\x1e\xc0h\x08E\xdc\xa3\x03\x91RD:^\xb48.\xdaM\x10\x1aC\xa44J\x1b0\x1a\xad\
\x14\x8d0\xb0\xdcU:\x9eU#\xb4F\x1am\x01\xe0"\xdd$\x9e\x93\x8c\xff\xac\x89"\
\x852\x82(\xeel\xad\x08K!\x01z\xd1\x03\xda\xc6@d\xccR;\x13I/\x01$b\x10\xbe+\
\x8c\xa9 \xce\xd5\xe6\x08\x10\xd8l\xa4\x96\xb924\x10j\x03\x8e\xcb\\\x08\xf5P\
1\xd0\x92&i\x02N\x95j\x84\xa6`3N\xa4(\x06\n<\x15{\xc0P\x8a\x14\xd5\xb8\x02\
\x04ZS\x0c"\xbc bff\x9ek\xfb}\x00\xe6\xeb\x01M#q\xd0K\x1e\x88\x8c-\x9a\x81!\
\x8eM\xf0t\x10b;\x08\x1fp\xa40z\x1a 4\x06\xad\xad[\x97\xae\xc6,\xb1R\x1b\x8d\
2\x1a\xcf\xf3\x98\xd5.\xdf\x7fv\x82|\xca\xe7\xd3\xaf]\xc3Z=\x8fj\xd6\x00p\
\xebE\x92\x95q\xd2\xd5\xa9\xa5\x91\xacL\xe1\xd6J\x00\xc8\xb0I\xa24\x89;?\xca\
\x8d\x1d\x11\xb7_\xb7\x110\xfc\xf0\xc4\x0c&\x99\x02)\x97\x8a\x9d\xa5\x98@cA\
\x00$\xc3Z\x03K\x12\t\x08W\x1au\x04\xe3\xa9\x86\xc1\x89\xe2@\x12\x80\xd2\xe7\
\x82\n\x13\xdf+\xfbX\xa6\xb5\x93\xbb\x9e\x99`[w\x0bo\xde\xbc\x8aW\xddr\x05\
\xd5\xc8v\xad\x9f\xda=\x80\xc8\xb4\xe0{^\x9ce\x04a\xa3J}a\x06\x03\xdc\xb0&\
\xcf\xce\x9b\xb7\x93\xcd\x15X\xd3f\xab\xf7\x17\xf6\x1e\xe5\xe73\x86|W\x9e\
\xf2tmqV\x941(m\x88\xb4&0\x06a\x8ci\xadN/\xc4\xff\xa0\x01\xe3\xfa*8\x189\xfe\
|S\xcb\x8e@\x98\xb8\x88i\xa28\xa016\x98B\xa3\t\xd1(ep\x92I\x8aA\x1b\xbf\xff\
\xc3\x93\xbcw\xcb\x1cW\xe6\x02\xb6\xad* \x04\x14\xeb\x01\x9e\x13\xa0\x8c\x83\
A#\xa4\xa4\xde\x08\xa97l}mF\x9aj=\xc0Kk\x1e:8\xc2\x03\xcfN\xf1\xf0DD\xae\xa3\
\x1b\\\x97\x00\x08c\x16\x99\x98:\xa1\x81\xba\x01O5\x9b\x1d\xe5\xb1\xb9\x98\
\xe9\x01\xa0\xdc\xae\xe2\xc8\x893\x1d\x97\x9c\xad!;\xc2e\xcd\xbbM\xa96\x06\
\x0c\xb6\'R\x06\x9b\xa3\r\xa4ry\x9a~\x82/\x9f\xa8\xf0\xb9\xb1\xd3\xfc\xcd5\
\x03\xdc\xb6z5\x9f}\xec\x0c\x0f\x8dE\x0ctwb\xb4\x05P,\x16ye\xa6\xc9\xc3;/\
\xe3\x91\xb3%n\xfd\xd11:V\xf5\xa2\x85Dyir\xddy\\\xcf\xa5\x1a(\x9aAH\xd6\x93\
\x80 \xd0\xd6\xa0M\x035\x03\xe9\xa0R\xe9(\x8f\xcfa\x1b\xb9:\x10\xc87\x1c\xfa\
\xd7\xa2\xd4j_`\xec\xf2?\xc2X+\x08P\xc4)\t\xbb\xf7\x13 \x080v\x18\x83\xf4}\
\xb2\x1d\x9d\xf8\xbd\xeb9P\xb2\xe8\xff\xe8\xba\x1d\x14\xf29F"\x8fq\x91bBf\
\x98\x93\x19\x8a\xc2\xae\x11\x9a8\xd4e\x96\t\x91b\x8e\x04\xa5H3:\xbf\xc0\xa9\
\xf1)\x9e\x1b9\xcb\x1b\xfb\x93\xec\xeak\xe5\xd0\xf8,g\xeb\x1a\xd7\xf7\xa8*\
\x1b\xcc\x85\xda\xec\\2\xacWc\xe5k@\xd3\x05\xb4\x1f5\xfe;p\x13\xb7\x94\xb5\
\x8d\x0c-\x05\r#\t\x16\x9b*\x03M\xa3p\xb4F.y)^\x11h\x85\x9f\xcd\xf1\xd0\xc8(\
\x0f\x1f\x1d\xe7\xc6\xcd=\xfc\xfc\xf7\xd2<|z\x8eH\x80\xeb\xb8\xd4*\x82\\\xd3\
>uy\x9b\xc7\x1dW\xe4(t\xe6\xe24\x1b\xf3]\x1b\xfa\xd2\x05~\xfb\xd2>\x10p\xc7\
\xde\x93\x14\xbd\x0c\x9d\x8eCQ\xdb\xa6\xb0w\xfe\xf4xl\xfd*\xd6\xdeMwhh\xc8\
\xfc\xe8\xee={j~v\xa4&\x9d~\x81F\x18C\xdd@3N\xbe&\xe6 \xc6 \xcd\xe2R\xe6\x1c\
\x10\xe1zT\xb3\xad\xdc\xf2\x93\x93|\xae\xd6\xe4]\xdbz\xb9\xf5\xca51H\x01\xba\
Ae\xcav\xab;\xbb\xb2\\\xd1\x97\xc3\xcd\xb4-\x19\x81\xa57jN\xcd\x96\xf9\xd4\
\x9e\x13<0\x16Q\xe8\xea\xa4\xa2\r5#H\x05\xd5\xca\x9a\x99\xa3#\xb1\xf5K\xb1\
\x07\x02\x17\xe0\xa6\xfd\xf7\x8c\x7f\xeb\x9a\xdb~X\xf73\x7fX1\x02\x11\x17\
\x0e\x85\xc0 \x08\xe2bR\x8d\xab\xf4\x85"\x8cFds\x8cK\x87\xf7\xed\x1d\xe7\x0b\
OO0\x90\xf6\x00\x8d\x90\x0e\xcdr\x99nY\xe7\x9b\x1fX\xcdC\'\xa6\xf9\xca\xdecd\
{z\xcf\x03 \x84\xa0\x19\x19\x9e\x9eo0C\x9aDW/U\xe9\xb2\x98\x19{\x17\x86\xcf\
\xe6\x1a\x0b\xa5\xd8\xf2\xc5\xd8\x0b\xd1\xe2\x92R\r\xcc\x9d\xf8\xca\x89\xee\
\x1d7k!\xb3\xc6X\xee\xab\xd8\x809\xdf\x81Fh+\xa4t\xb8\xe8\x0eV6\x0f\xa9\x0c\
\xfbj\r\xf6-\x84VA\xe9\xc0l\xc0\xd6\xbcm\x1a\xce\x04\x92\x9f.\xa4 \x91|\xfe\
\xf3\xd2\x81\\;$\x134Y\xec\xeb%\xae\n\xc3\x9d\xc3{\x0f\xc7\xd6/\x12/+\x01\
\xb5\x08@\xbf\xee\xc8\x83\xc7\xc6\nk\xbfUI\xb6|\xd8\xae\x1f=&*!S\x95&;W\xe5\
\xd9\x96\x87\xc3\xa3\x93\x90N\xaf\xac\xfcy.\x91 \xe3W+\x03B\xf1\xf1W\xdbu\
\xc2\xe9b\x00\x1d\x9d\xd0\xd1\x0ez\xb1\xea,\x17\x03Z\xdb\xab\xb0\xbb>\x1b\
\xa6\x0e\x1dk\xabN\xcd\xc5\xca\xcfpn\xdfT/\xed\xcc\r\x0e\x0ez\'\xbaw\x0c<\
\xba\xf9\xad\x8f\x84Nb\x1d*\x82\xd13|xk\x86/\xbf\xedrN\xcdW\xb9\xeb\xc9\xb3L\
\xd4\x14\xd2qx!b\xb4\xc1\xd5\x8a\xddk\xf2\x0c\xeeZ\xcd\xb1\xe9"\xbb\xef\xd9\
\xc7t\xaa\x0b\xd2\xa9\x15\x94_n\x04\x01\x8eC\xb6Q,\xbe\xfb\x97C?H\x84\xf5I\
\xe0\x04p\x008\x06\xcc\r\r\r\x85\xcb\xf7\x85\xa2\x8d\x93\x07\xa6\x86;\xb6|r\
\xb8s\xcb?k\xd7\xf5\xe8\xec\xe6+\x07\xce\xd0\x91>\xc4\xa7\xaf\xdf\xc2\xdf\
\xdd\xb8\xe5\x05)\xbe\x92\xfcrd\x96\x0f\xfc\xc7\x11\xa6E\x16\xd2)\x1bg\xe2b;\
GVy\xa9\x95z\xcd\x89\x1f?\x9e\x08\xebe\xec\x12r\x1c\x98"\xe6?\\\xb07:88\xe8\
\x01\x85\x07_\xf9\xc1\xbf\x98j\x19\xf8\x13\x00\xca%\x98\x9dbG\xab\xe0\x8d\
\xab\x0b\xb4xr\xc5\xb3\x80\x15E\x80@rx\xa1\xc6\x83\xcfUh\xb8yD\xa1-nO4\xc8\
\x0b\x0eF\xec\xde\r\xb8\x0e\x08\xc1eg\x1e{\xea\xaa\x93?\xd9\x17+}\x1c8\x04\
\x9c\x02\x16\x86\x86\x86.\xd8\x17\xb2\x12\x01\x95w>\xf5\xf5\xbf\x7f\xe0U\x1f\
\xea\x9d\xc9\xf6\xfc\x0e\x99\x1c\xf8\t\x0eT\xca\x1c8P\x8d\x1b\xa2\x17!B\x80\
\x9f\x84\x96\x1e\xf0|\xcc\x12\xc7y~2X\xa6\xfc\xa6\x89\xfd\x87\xaf:\xf9\x93\
\x03X\xde\x8f\x01g\x80\t\x16\xeb\xed\xe2#\x17\xeeN\xc7{\x8f\x19\xa0\xf7\xc1]\
\x1f\xfc\xccTK\xff\xbb\x01\xab\xf8K=\xd14\x00z\xd9V\x87\x89\x0f\n\xc4R\xa9@J\
p\xad=7M\xec?|\xfd\x91\xef=\x81\xcd\xf7#\xc0\xb3X\x0f\x8c\x01\xd5\xe5\'7+n\
\xaf\xc7T\xca\x01}?\xdeq\xf3\xc7G\xda6\xbcWI\xc7\'R\x16\x88\xd6\xe7\x9fT\xbc\
T1q\xa6q\x1dp\x1d\x1c\x1dE;F\x1e\xff\xd5\x95\'\x1f9\x18+?\x8a\r\xdc\xe3\xf1\
\xe7\xf2"u\x16\xe5y\xdb\xeb\xb1DXW\x8d\xbf\xf1\xc0\xbd_zf\xcd\xb5\xa7\x0e\
\xf5_\xf9\x91\x9a\x9f\xebCJ\x88"\xe2\xf5_l\x86\x17\xa3\xf4\xb2g\x1c\xc7Z]J\
\xf2\xf5\xb9\xf9W\x9f|\xf8\x89\xb5\xd3\xcf\x8e`i3\x8e\xe5\xfb\xc9\xf8\xf3y\
\xd4Y\x94\x8b\x9e\xd0\x0c\x0e\x0e\n\xecZ:\x0f\xf4,\xa4;\xb6\xfdb\xe3\x8d\xb7\
\x8c\xb5\xae}]$\xfd$Z[\x10\x91\xb2\x01\xf9B\xd8%8gq)AJ<\xdd\x0c\xd6O\x1e:v\
\xd5\xc9G\xf6\'\xc3Z\t[\xa4\xc6\x80\xe1x\xbc\xf8\x13\x9a\x15@d\x81N\xa0\x7f\
\xa4m\xc3\xae\x83\xfdW\xbd}&\xd7\xb3\xbd\x96\xc8\xb6\xda\x05\x9b\xb6\x05k\
\xa5\xe0\\\xa4\x9a\x94\xf6 MH\x10\x90i\x94K=\x0b\xc3\xa3\x97\x9dy\xecH{eb\
\x16k\xe1Y\xce\x05\xec\x08/\xe7\x8c\xec\x02\x10.v\x11\xdd\nt\x03}\xd3\xb9\
\xde-g\xdb7^6\x99\x1f\xd8\\J\xb7\xf5\xd4\xfdL>t\xfc\xa4Y!8\x04\xc6x*h\xa6\
\x83r\xa5P\x99\x99\xed]\x18\x9e\x18\x98;1\xdeR\x9b]\xac\xa8\x8b\x15v\x1c\xcb\
\xf5I~\x13\xa7\x94\x17\x00q\xb0\x0b\xe9,\xd0\x86\xf5H7\xd0^I\xb4\xac\x9a\xc9\
\xf7\xf4\xcfd{\xfa\x8a\xe9\xb6\xae\xa6\x9b\xcaF\x8e\x97\xf0T\x10&\x83Z\xa3P\
\x9bY\xe8,\x8f\xcd\xb5U&\xe7\xd3A\xa5\xc2\xb9\x05I\x05K\x99\xd9X\xe9il\xc1\
\xfa\xcd\x9d\x13_\x00b\xd1\x1b\tl\xaam\xc1z\xa5\x10\x7f\xce\xc7\xdf\xa7\x80$\
\xf6\x18Y\x12\xefQ\x01\x8dX\xf1*\x96\xd7\x8b\x8d\xd9<\xe7:\xcc\xdf\xfcI\xfd\
\xaf\x01\xe2\xc7\xca\xa6b\xc5\x97\x8fd\xfc\xf7E\x00A\x0c\xa0z\xc1\xa8\xc7\
\xe3\xff\xf7\xb7\x12\xbf\x06\x88\x8c\xc1,\x02\xf2\xb1\x1e\xf2\xb1\x1eXlxT\
\xacd3\xbe\x06\xd8\xb4\xf8\xb2~\xad\xf2\xbf\x96U\x1c\xbb\x8f\xc1@\xc5\x00\
\x00\x00\x00IEND\xaeB`\x82\x0f\xa4{\xe2'