from core import models
from core.translation import _t

def dropShadow(image, x_offset=5, y_offset=5, back_colour=(255,255,255,0), 
                shadow_colour=0x444444, border=8, iterations=3, 
                force_back=False, cache={}):
    """
    Add a gaussian blur drop shadow to an image.  

    image           - The image to overlay on top of the shadow.
    offset          - Offset of the shadow from the image as an (x,y) tuple.  
                        Can be positive or negative.
    back_colour     - Background colour behind the image.
    shadow_colour   - Shadow colour (darkness).
    border          - Width of the border around the image.  This must be wide
                        enough to account for the blurring of the shadow.
    iterations      - Number of times to apply the filter.  More iterations 
                        produce a more blurred shadow, but increase processing 
                        time.
    """
    #get info
    size        = image.size
    mode        = image.mode
       
    back        = None
            
    #assert image is RGBA
    if mode != 'RGBA':
        #create cache id
        id  = ''.join([str(x) for x in ['shadow_',size,x_offset,y_offset,
                    border, iterations,back_colour,shadow_colour]])
                    
        #look up in cache
        if cache.has_key(id):
            #retrieve from cache
            back, back_size = cache[id]
            
    if back is None:
        #size of backdrop
        back_size       = (size[0] + abs(x_offset) + 2*border,
                        size[1] + abs(y_offset) + 2*border)

        #create shadow mask
        if mode == 'RGBA':
            image_mask  = image.split()[-1]
            shadow      = Image.new('L',back_size,0)
        else:
            image_mask  = Image.new(mode,size,shadow_colour)
            shadow      = Image.new(mode,back_size,back_colour)
            
        shadow_left     = border + max(x_offset, 0)
        shadow_top      = border + max(y_offset, 0)
        shadow.paste(image_mask, (shadow_left, shadow_top, 
                                shadow_left + size[0], shadow_top + size[1]))
        del image_mask #free up memory
                            
        #blur shadow mask

        #Apply the filter to blur the edges of the shadow.  Since a small 
        #kernel is used, the filter must be applied repeatedly to get a decent 
        #blur.
        n   = 0
        while n < iterations:
            shadow = shadow.filter(ImageFilter.BLUR)
            n += 1

        #create back
        if mode == 'RGBA':
            back  = Image.new('RGBA',back_size,shadow_colour)
            back.putalpha(shadow)
            del shadow #free up memory
        else:
            back        = shadow
            cache[id]   = back, back_size

    #Paste the input image onto the shadow backdrop  
    image_left  = border - min(x_offset, 0)
    image_top   = border - min(y_offset, 0)
    if mode == 'RGBA':
        back.paste(image, (image_left, image_top),image)
        if force_back:
            mask    = back.split()[-1]
            back.paste(Image.new('RGB',back.size,back_colour),(0,0),
                ImageChops.invert(mask))
            back.putalpha(mask)
    else:
        back.paste(image, (image_left, image_top))
    
    return back

class Action(models.Action):
    """Drops shadow"""
    
    label       = _t('Shadow')
    author      = 'Stani'
    email       = 'spe.stani.be@gmail.com'
    version     = '0.1'
    tags        = [_t('filter')]
    __doc__     = _t('Drops a blurred shadow under a photo')
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Horizontal Offset')]        = models.PixelField('5')
        fields[_t('Vertical Offset')]          = models.PixelField('5')
        fields[_t('Border')]                   = models.PixelField('8')
        fields[_t('Shadow Blur')]              = models.SliderField(3,1,20)
        fields[_t('Background Colour')]        = models.ColourField('#FFFFFF')
        fields[_t('Shadow Colour')]            = models.ColourField('#444444')
        fields[_t('Force Background Colour')]  = models.BooleanField(True)
        
        super(Action,self).__init__(fields)
        
    def apply(self,photo,setting,cache):
        #get info
        info        = photo.get_info()
        #size
        x0, y0      = info['new_size']
        dpi         = info['new_dpi']
        #dpi
        parameters  = {
            'x_offset'      : self.get_field_size('Horizontal Offset',
                                info,x0,dpi),
            'y_offset'      : self.get_field_size('Vertical Offset',
                                info,x0,dpi),
            'border'        : self.get_field_size('Border', info,x0,dpi),
            'iterations'    : self.get_field_value('Shadow Blur', info,),
            'force_back'    : self.get_field_value('Force Background Colour',
                                info),
            'back_colour'   : self.get_field_value('Background Colour', info),
            'shadow_colour' : self.get_field_value('Shadow Colour', info),
            'cache'         : cache,
        }

        #manipulate layer
        return self.apply_to_current_layer_image(photo,dropShadow,**parameters) 
    
    def import_modules(self):
        #lazily import
        global Image, ImageChops, ImageFilter
        import Image, ImageChops, ImageFilter
    
    description = _t("""
""")

    icon = \
'x\xda\x01\xfd\x0e\x02\xf1\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\
\x00\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\
\x08\x08|\x08d\x88\x00\x00\x0e\xb4IDATh\x81\xb5\x9ay\x94V\xc5\x99\xc6\x7fu\
\x97ok\xba\xe9\xa6\x9b\xa5Y\x04\x01\x05E\x10\x17\x8ck4\n\n\x89Q\xe3\x98L\x8e\
1n\xd1\x99xrL\xcc\x84\xc8,\xc9\xc9\x98\xe5\xcc!\xa3\x89\x9aq&n\x89\xe3\x12Gc\
\\\xa3\t\x9a\xb8\x81#\xb2\x8a\x88\x12\xa0\x91\xa5[h\x9a^\xbf\xfe\x96{oU\xbd\
\xf3\xc7\xbd_w\x83\xdd\x88s:uN\x9d{\xfb\xbbuo=\xcf[\xef[\xf5\xbcU\xadD\x84\
\xbfF\xf9\xe6W\x1b\x8e\xad\xadrgD\xda\x96^|\xa3w\xdd\xda\xf7K\xfb\xfe*\x1d\
\x89\xc8\xb0\xd6\xb93\xb3\xf5\xcf\xfc\xd7\x94\xa5\xb2uN\x97|0[d\xfbq\xd2\xfe\
\xea\xf4\xcd7~q\xe45\xd9\xb4J\x0fw\x7f\xdep\x1a\xa3\xb6\xdaUo?u\xf4\xcf\xa6\
\xccH_Ig\x04\xda\x82\xb5\x8c\xaaQ3\xee\xbce\xf4\xaf\xc2\xd0z\xc0\xbd\xc3\xd9\
\xa73\x9c\x1f\xbb\xf1\xf2\xfa\xab\xa6\x1c\x9d\xbe\x92\x8e\x08\xb4\x01\x93\
\xd4\x92\x81\xbc\xe6\x9f.\xaf^RS\xe5N\x1b\xce>\x87\x95\xc0\xf5\x97\xd6]Lobyc\
\xfb\t\x18\x0be\xc3\xe4\t\xce\xb4E\xf3R\xa7\rg\x9f\x87M\xa0v\xa4\xe7\x1e\xea\
y\xcd\x08\xd7\xc9z\xb6\x96\xc8\x80M@\x1b\xe9\'\xa2-X\xc3\xa8\x11\xce\x98C}gd\
N}"\xa3\x1e2\x06\xa6L\xca8\x0b\xcf\xad[\xf8\xed\xeb\x1b/\xdc\xb1\xe2\x84cW>=\
\xab\xf5\xf1\xe7\xdb_zfY\xe7\x1f\x9bv\x05\xcd\x074\x160\x91\xb1\x18\xfaG\xc0\
&\xb5r\xaf,\xc6\xd8A\xa7\xbdKNvO\xfe\xfcI\xce\x85[\x7f\xea\x9d\xb9\xe7\x0e7|\
\xe1m\xbb\xfa\xc17x\xf2\xb5-\xb2\xe1\xffE`\xe2\xf8\xb4\xbf\xe2\xc9\xe3\x1e\
\x9823w9e\r\x91\xe5\xd4\x93\xab8\xf5\xf4\xdc\x97\xbe\xf1\xa5\xba\xf5\x13\xc7\
\xa6\x165\xb7\x86\xad}\xf8\x05A[A\xd3\x17\xbc1p\x03\x06\x10\x0bJ\x10\xf3\xd1\
y\xfb\xae\xab\xfdEO\xdf\xec=\x8a\xb2#)\xc5#\xf7\xb5\xcf\xb0\xe8\x8ay\xdc\xf8\
wg\xaa\xbf\xbfg\x85\xfcv(\x9cC\x0e\xd7\x85\xe7\xd5\xcd\x9f2#{9\xf9\x08J\x1a\
\x02\r\x85\x08:B\xa6MO\x9f\xf0\xbb\xdb\'=4\xb915\xa1\xd2>_4\xa2D\x14"1\xd8\
\x03\xac\xaf\xfb\xe3A0\x03\xfb\xb9i\xa1w\xd67\x16\xb9\xbf!0#\xc9\x1b$\xb0\
\x98@\x88\xf2\x90\x86\xba\x1f}\x8e;\x8e\xacW3>1\x81\x9b\xae\x1dw\t\x81\x86\
\xb0\xe2\xcfI\xd5\x16z4\x9f:9\xb3\xe0\x89\xa5\xe3\x1ehl\xf0\xc6\x9e1\xb7j\
\xec\xfd?l\\R?J\xe6\xa2\xcb\x80\x01U\x19\x1a\xe2\xf7\x82\x00\xf2\x05\xbey\
\x01_\xfd\xf2\xe9\xde\x82\x9a\xacJ\xfd\xe2J\x7f\xe1\xedW\xbb\xcf\x10\xe8ZB\
\x8bh\xc1j\xd0\x1aL\x04\xc5\x12\xd4gh<\xe3H\x16\x0c\x85sP\x17\x1aY\xed9;W\
\x1c?\x93rb5;\x80\x80$\x81\xd9e\x9877;\x7f\xe5\x83\x93_\x1f]\x13\xe5r\r\xfeD\
\xf4\x0c\x181\x07\xd2\xf5\x88\xe7!\x12\x00e0y(\xb5\xc1\xfe&f\xa77\xcf\xfb\
\x9f\xafw.\xdb\xbd\x87\x8d\x93\xc6\xca4\x02]\x85\xb6}\x83\xd4w5`4\xa4<\xa8\
\xcf2^)\xa5d\x10\xd90(\x81(\x12\xd9\xb2\xad\xd8z\xca\xbc\\\x7f\x00\x1a\x13\
\xcf*\x95{\xd7\x05k\x99<9u4\x99\xd3\xa0\xfa\\\xa8;\x15\xf1\x1c\x84<\xa8<B/P@\
(!\x14a\xeaLL\xf1\x04\xbc-k\xd4\xa4\x9a\xbf\xcc1\xbb\xbb\xb0\x80\xf2b\xaf3:!\
\x90x\x9d5\x10\x18\xe8,P"\x1e\xd3\x8f\x10\x18\xd4\x85\x8ae#\x8f?\xdf\xfe"\
\xbe\x0c\xb0\xbe$\xd3\xa1\x06\xcf\x05_\x83?\x0e\x1a\x7f\x08\x93\x97B\xc3\x02\
\xc4\t\xc0\xec\x07\xdb\x0e\xa6\x13l\x17b\xbb\xc0v\x81t \xd2\x83\xe4|\xc2\xb9\
\'\x10.Z\x80>f*A\x11\n]\x10\x05\xfdV7\x11\xe8\x08\x94\x85B\x19\xd6\xb6\xb0ND\
\xec`X\x87\x8c\x81\xa7\xff\xdc\xbdl\xfb\xdb\x85\r\xa4\x06.H\x06|\x0f|\x01\
\xffH8\xe2_\xa1\xf6<\xa0\x84\xd8v\x90"\xa2J\x08eD\x95\xe3+eD\x95bWR\xa5xT\
\xa4\x17<\x85:g\x0e\xeeY3\xd1\xa1\xa2\xd4\x1d\x930\t\t%\x10\x86Dw\xbd\xc1\
\x03\x9b\xdbX=\x14Nu(5:\xb915\xf6\x89\x7f\x1f\xff\xd0\xbc\x932\x0b\xe8\x8ab\
\xcbg\x04\xbc\xc90\xe56H\x8d\x03\x9b\x07UD( \xaa\x00\xc4\xf7\xa8B\xdfoB1v%U\
\x00J(\x1d\xa0\xa2\x00l\x80r4\xc1\xba\x9dt?\xdb\x04\x08\xa9ll\xd5\x14\xf0\
\x0f/p\xfb\xbd\xabY\n\xb4\x0e\xe6\xff\x87\x1c\x01\x80\x9d{\xc2\xd6\xcb\x16\
\xb7\\\xf3\xd6\xca\xde\x97H+\xf0\x15\xa8:\x18\xbf8\x01\xdf\x0b\xaa\x8cPDT1\
\xbeRL,]\x04U\x84\xca\xdf\xaa\x04RF\xe9\x10\xa5\xc3\x18\xbc\x0e\xa0\\${\\-\
\x99\x13GQ\xea\x86r/\x94\x02\xc2o?\xcf\xed\xbfZ\xcb\xad"\xb2w(\xf0\x1fK\x00`\
\xe7\xde\xa8\xe5\x0b\x8b\xf7\\\xf9A\x8b\xddDJ\xc1\x98\xeb\xa0\xfa\x14\xb0\
\xc5\x04|\xe2\x1e\x95\x9a\xfc6\xd0\x85\xfa\x9e\x99\xb0\xaf*\x13\x82M\xc8\x04\
ej>U\x8b\xdf\x98Bz\xe1\xb1u\xbct\xdf\x1a\x96j#-\x1f\x87\xef\xb0t\xc7\x87\xed\
f\xafE\x87dfA\xdd\x05\xb1\xe5I\x80\x12[\xd7q\xf3\xb8n\x17\xaeSL@\x97 y&\x94P\
&$\xedt\x93r;q\xe4@2\x12\x05x\x19M\xed)9\xa2\x10\x8ay\x02`\xff\xe1`\x1b\x94@\
M\x95\xeb\xc4\xd5q\x00\xbe{U\xc39\xd3\xa6{\xb3\xc9\x9c\x02n\x16\xa4\x12\x98E\
\x94S\x04\xd5\xcb\xba\xed\xa3yq\xe3L\xda\xf2\xe0\xa8\xde>\xf0PB\xd92\xca\x94\
xs\xf3\xd9\xbc\xbc\xf1\x12\x8aE\x17\xc7\x96A\x07`\x02\x94\t\x91r\x99\xdc\x04\
\x8b\xcd)\xe6\xd6rRm\x9a#F\xa4\x94\x1a\x91R\xce\x88\x94r\xaaSJ\r\x86\xf5\x80\
 \xfe\xde\xb5\xa3\xae\xb9\xee\xa2\xea\x859\x8f\xd1V\x1b\xc1Z\xc4\x88\x1e\xd9\
\x90\x9a\x91\xab\x1f=\x99\xe9\xf7@\xaa\x0e\xa1;\t\xd4^\x1c\xb7\x97\xcd->\xcf\
\xaf\xff,om\xf58f\xd2\nn\xb9\xeci"B,E\xb0%|)\xd0\xdcV\xcf\xb3\x1b\xee\xe6\
\xad\xad\x1e_\x9c\xbd\x84\x8b\xe6<\x85)91\x01\x1d\xa0t\x19\xe5\x95\xf9\xf0\
\xb92;_\x14\xf2i\xd6\xa5<:\xc5\xa0\x8c\x01\xc7"\xeb\xdb\xd8\xf2\x87\x9d<\xf6\
\xa7\x16y\xad\x82\xd9\x038\xf6\xc8t\xc3sw\x8c\xbf\xf5\xc7\xdfi\xb8\x8a\xa2N\
\xd4\xa4\x93\x880\x81\xac\x86\xf4q\x90\xaaG\xe8\xa1\x7fJ,\x03\x05\x9a\xf6M\
\xe3\xd5M\x1e\xbf\x7f\r.<g\x14\x16\x1d\xbb\x90\x84(\x13\xe1\xb8\x05\xdeo>\
\x9d_\xbf\xea\xb1n=\xcc\x189\x91\x8b\x8e/\xa2t\xaa/\x1e\x94\tAB\xaa\xa7\n80\
\xce\xe7D?\x13\xaf\t\x95\xa9\xf5\x82\xf1\x9cwV=\x97_{\x94\xfa\xfe#\xdb\xb9\'\
0\x12x\x00\xb7\xdeX\xff\xddiS\xbd\xab\xe8\x8c\x0e\x94\r6Y;|\x81\xecQ\xe0\nb\
\n\xc9\xecRJj\x81]\xfb\x85\xdf/\x8f\x9b\x1a\xe8\x0blU\xf1s\xa7L\xf3~\x8fu\
\xef\xc6m\xb4U`\x02\xb0\x80\xad\xc4B\xbc\x08T\x8d\x07\xb7\n\xf2\xdd\x90S\xfd\
\xab\xb36P\xd4\xa0,#o\x98\xce\x9dM=\xec\x03\x1es\xbe<\x7f\xc41\x9f\xfdL\xee\
\x06\xba\xa2\x03\x17,c\xe2\xb7\xac\x89G\xc3K%nS\xec\x0b\xde\x98H\xc8\x9936\
\x93\x9b\x16\xc2Yp\xfc\x05\xdbq\xe9AL\x94X6\x80\x00N=b\x15\xdet\r\xf3a\xe6\
\xec\xcdP2\xa8\xc4\xff\x95)\xc7(\x05\x1c\x1fRu\xa0\xc3x56:\xb9&\xf7\x91\x89\
\x9db\xe1x\xfe\xd6u\xd4\x08o\xc6$\xff\x18\x94\xad\x8e\xc1V,? \x93\x12\xc0\
\xb8 \x1a\x19h\xf9\x84\x88\x96\x88\xd9G\xec\xe0\xee\x9b\x1f\xe7\xadt\x03\xdf\
j\xf8#\xc6\xda\xa4\xd7\x00L\x88\xb1\xc2\xac1\x9b\xf9\xb7kn\xe3\x83\x91\r\x9c\
\xef\xbe\x0c\x81\x03:!h\xa2\x01Q\t\x8e\x9bh\xa3\x04\xb4M\xecY)Z`J\x15G\xf9\
\x8a\x06/\x8al\xa9\xdf\xda\x89\x96\xd7\xd2\xefB\x02\x18\x8d\xd8x\xd6\x89-\
\x9f\x8c@2\x12\x11\x86+&\xfc/W\x90\xc7`\xd0bq\x88@"\x94D\x88\x890bY<\xf6?!*B\
\xa4\x10kP6\x88]\xe8\xa0"\x89\xe8\xd5\x89S\x1c\xac\x82\x14P2\x84F\x10\xef\
\xc5\xd5\xe5u\x8b[\xc2m\xa3\xaa\x99Nd\x0fJF\x0cX\x05Q\x04Q7B"\x8fI\x02X\x05\
\xa0"Dw\x13\x9a<H\t\x15\x95q\x83b2\xbb\xc4+.:D\xd9\x00A\xa3,\xa0\x03\x14!\
\xd8(F:\x00\x99\r\xa0\xdc\x11\xdf\x1b}\xe0\xe3\nx\xdf\x855\xed\xbc\xa3\x85No\
\xcd_\x82\xd6\x1b/\xaa\xfa\xc9/\x96T\xff\x9a\xc8B0 \x88\x8d\x05\x04\xb4\x03\
\xf9\x1d@\x0f\xe2\x94@\x92QP\x01\xce\xf6\xb5x\xabV@\x18&I\x8c{\x90\xc9d\x80\
\x08\xb6qB&\xa0\xaa-L\x15\xf0\xe9\x7f\xeeBi?\x94;A%\xe2y\xe0\xe4\xaf\x14d\\\
\xd8W\xa6\xf0\xfc\x87<\x01\xf4z\x00\xf7/+\xfd&\xd2\xd6\xbf\xf9\x0b\xe9\xc5S\
\x1b9\x1a\xd7\x92L\x11I@\xfb\xa8\xd6\x8dPjF\xb2\x1e\x8a\x12\xa8\x08w\xc7zx\
\xf55\x9a\xa7\xfe\x18\xaa\'\xe2{\x1en\xba\x06\xcf\xf7\xf1\\\x17\x95\xac=\xc6\
Z\x8c\xb1\xe8\xa0\x17\x13\x16\x08\xb4Kn\xff\x9f\x19\xd3|?L)\xf7#ta\xff\xc6X\
\xa5d\xb2\xb1\xb9\x07\x12(\x19\xa2\xe5\xfbx\xf3\xee\xad\xdc\xb9\xab\xc0+"b=\
\x80b`C\xe0\xde\xea\xaczy\xd1\x89\xde\xa9\xf5Uj\xac1V\x10\x10c\xc3\xd3\x8e\
\xe5\xb8k\x17\xec\xfd\xba\xf3\xe1\x1a\xec\xb4y\x89\xc6\xd1\xa8\xfd\xbb\xd1&K\
~\xdc\xe7\xf0\xb35\xe0\xbb\xa8\x94\x8f\xa4|\x94\xef\xa3\x1c\'\xb6\xa2\xd1H\
\x14\xa1\x83\x900\x8a(\xea\x0c~\xf4!t\r\xd8\xa9qb\x85\xb2\x7f\x1dtY:\xfe\xfb\
=\xfeC\x0b\x1d*V\x0bJ\x81\xbc\xdf\xc3\x8emy\xde\x05v\x88H\x04\x07ed\xf9\x924\
\x01M\x1cTj\xaa\x9c\xda\xf3g\xc9\xd9\x13\xc7l8\x86i\xc7 *^\xc0\xf4\xd1\xd3\
\xf1\xb7\xbcO]\xebs\xf4N\xbb\x0e\x87\x02.\x82\xaf\x1c|\xc7\xc5\xf3\x1c\xac\
\xb5D\xd6`\xd1\xb8D(\x14\xd9`\x07\r\x85\xa7`t\xb1?\xcfJC\xcf\x16\xd0m\xb0\
\xbe\x9bU\xcf4\xf3s\x11\xe9:\xc0\xff\x07I+\x0fK\xcc\xf5\x14l\xd7\xb6\x16y\
\x8f\xbdM\xa8\x96\x8d@\x14k\xfd\x9a*\xecis\x19\xdd|+5{~\x07n\x06q\xb2\x95\
\xce\x06T\x00\x858Y\x1c\t\x19\xd3~/\xe9\xccj\xc8&\xf1\xe1\xc4n\xd3\xbc<^\x0e\
6\xf7\xf0\xd6\xc1\xe0\x01\x06\x93\xd5\x87E\xe0\x8e\xcb\xd4\xa23\x1a\xcdy\xd2\
Y\xc2{\xf9\x19T\xe7N\xc0\xa2\xa2\x02f\xd2\x04\x9c\xb9u\x8c\xdd\xfa-\xea6}\
\x07\xb7\xe7=\xb4\xca\x11I\x8a\xd0:D\xe2\xa1\xc9\xa1\xc9\x90\xca\xafe\xec\
\xee\x7f\xa4J=\x01u\xf9\x18\xbc\x02\xb2\xb0k\x19\xec]\x05\xd6\x83\x8b&rICF\
\x8d;\x1cl\x87\xcc\xc8\x00n<G\x9du\xe7e<[*S\x8b\x0b\x19\x1f\xcc\xa4Fd\xe1\
\x19(GCX\x044\xee\x9e6\x9c\xa6\x16DM$\xac\xff<\xd4\xccD\xd2\xb5\xa0\\$\xeaA\
\x15\xdf\'\xd3\xfb,8\xbb!\xdb\x01\xd8dN\x84\x9em\xf0\xee}\x10t\xc5\t~\xc6\
\x85\xb7;y\xe9\x07\xefpMk\xe9\xd09\xc1!\t\xfc\xfc2\xb5\xe8\xa6\xb3y\xa4\x14P\
\x17F\xf1\xf0\xfa\xe98p\xec\xa4\xd1\xf8\x8b\xe6\xe0d\x80b\x11%\x01DeT\xb1\
\x88\xd3Y\x80(\r\x92\x01e\xc0\xc9\x83\xd7\x01\xbe\xed\xf7y\x17HC\xf7&\xd8\
\xf0K\xd0%p\x06D\xe4\x08\x0f\xd6v\xf0\xa7\xefm\xe0+me\x19\xf2pdH\x02\x0b\x8f\
U\'\xbep\x03\xaf\x05\x01#\x820^\xceu"\xac<?\xee_\xc6\xd4\x90>\xa9\x91\xec\
\x8cj0e\x08\xca\xa0c\xedO\x14\xa0\xa2D\xe3T@+\xfa\x12^\xd3\x03\x1f\xbc\x10\
\xbbM\x98?\x10|\xa5Ty\xb0\xbe\x93\x95K\xd6sIg \xad\x1fmq\x88\xbd\xd1Kgs\xa9\
\xb2\x8c(\x07\x03v\x06\x13]\x12\x96\xc1\xf5@m\xef\xa1\xb8\xbd\x87\xf2\xf1\
\xf5d\x8f\xce\x91n\x10\xdcT\x08J\x03!\xa0c\xa6Nbq\ra\x07t\xed\x80\x0f\x97C\
\xdbF\xf0\xd2\x83\x83\x87\xf8XaN-\xa7\xce\x1a\xc9\xb9\xc0\xa3\x87M\xa0&\xa3\
\x9c\xf5\x8b9\xa3P\x88\x05UEL\xf5m<%$\x14\x90\xf5\xa0\xbc\xb2\x9d\xe2\x9b\
\xed\x04U\x1e\xb9).\xb9\tBfT\x84\x9f\x8b\xdf\x0f\xba\xa0\xb07^\xa4\xf2\xdb\
\xc0)\xc5\xd2+;"N=\x86*V\xc0U01\xc7\xb1C\xb5\x19\x94\x80\x15\x08#\x82\x81V?\
\x98\x80c\xa1\x1c\x12=\xbc\x99g\x9bzXs\xd6x>=\xb7V\x7f\xba\xa7IWi@T,\xa7\xb4\
\x8e%\x8f)\xc7{=m\x9a\x96U\x9d\xacl\xca\xb3\xea\xd2#\xf8\xcaQ\xd5\xcc)\x9b\
\xc1P\xf4\x17#(\xa5\x943\xd8\xe6\xd6\xa0\x04z\x03\xb1w^\xacV_\x7f\x12\x17\
\x14*\x04t\x7fU\xc9\x8bK\xdf\xe2\xae\x077\xf1S\x11\xd9\x93v\xd5\x1dO\x9c\xcf\
K\xe3\xb3\x9c\x11\x05\xfd\x99T\xa5\xb8^\xbc\xcf\xf9\xf0&~\xf9\\3?\x13\x91\
\xe2\xd8\xacz\xf0\x969<t\xf2(\xe6\xf7\xea\x8f\xe2pT,\x8cw\x17\xd8\xf9\x89w\
\xe6\x1e^\xcb\x13\x9d\x05\xf2\x9e$IE\x02\xde\xb1\x10\x85\x98\x7fy\x9d\xdb\
\x1fy\x8f\xdbDd\x0f@`\xa4\x84\xa6\\ID\xac\x80r\xfakE\xd7XK\xaf\x88\x14\x01ZK\
\xb2\xf7\x07\xefp\xf5\xda\x0e^\xcay\x89\x80\x1b\x00>\xe7\xc2\xf2},\xdf\xd8\
\xcd\xcbC\xe1\x1c\x92\xc0\xaaf\xd9\xb0d\x19_\xdb\xd7\xc3n_\xe2\xf4\xce\x07\
\xc2\x08}\xdf\x06\x1e~\xf8=\x96j+}\xa74U\xber\xacA\xe9A$\xf0\xc0\xa2\xd4\x01\
\xfa\x8c\xd6\x92\xb4\xfc\xf3\x06\xae\xd8\xd0\xc9\x9bi\'\x9eyr\x1ex\n^i\xe5\
\xf5\x1fmdIQ\xb3k\xa8\xef\x1d\xf2\x88\xe9\xa1\xb7\xe5\xb7G\x8cT\x1bNid~]\x86\
\t]eJ\xef\xb6\xb1~s\x07\xab\x81\xb6\x83\xdb[\x83\x1a|\xa0\x0f\xe4p\xf0\x0f\
\xede\xd9W\x93R\x17\xcf\xa9\xe5\xdc\x899f\x19\x81\xddEv\xbd\xdd\xc1+\x81eWE\
\xb8}b\x02\x00\xbb\xbae\x0b\xb0E\xc5\xdaX\r\xe5\x8b\x08\xe2\xaaC\x1a?F\xaf>\
\xbaE\x0e\xd0\x13J\x1b\xf0X\xdcf\xf0\x80\x1d\xac\x1c\xf6\x89\xa0\xc4e\xc8\
\x8f\x16\xb4\xc8\xa6n\xb6f\x868\xcbT\nJ\x06\xfb^7\x1f(5\xf8&\xd5\x80\xbe\x0e\
\x0b<\x0c\xf39\xf1\xf3-<\x1aZ\xba\xddA\xe0e\\X\xdb\xce\x1bMy6\rg\x9f\xc3J`\
\xc5>y\xed\xbem|?\xe3\xc63\x88\xef@\xda\x89u\xcd\xfe\x80\xfc\xdd\xdb\xb8\x83\
8\x19\x19\xb6\xff0\xf9X5\xfaI\x8b\xef(\xff\xcc1\xfc\xcd\x82F.\x9b\\\xc5Q\x81\
!X\xd3\xce\xc6?\xec\xe1\xc9\x1d\xbd\xbcR\x99B\x87\xab\x0c;\x81\xbe\x0f+U\x95\
v\x18m@\xb4\xa5\x0b\xe8\x19N\xcbW\xca\xff\x01\x8dv\x07j\xce\x12)\x06\x00\x00\
\x00\x00IEND\xaeB`\x82\x94r=,'
