from core import models
from core.translation import _t

METHODS = [_t('By offset'),_t('Tile'),_t('Scale')]

def reduce_opacity(im, opacity):
    """Returns an image with reduced opacity."""
    assert opacity >= 0 and opacity <= 1
    if im.mode != 'RGBA':
        im = im.convert('RGBA')
    else:
        im = im.copy()
    alpha = im.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    im.putalpha(alpha)
    return im

def watermark(im, mark, position):
    """Adds a watermark to an image."""
    if im.mode != 'RGBA':
        im = im.convert('RGBA')
    # create a transparent layer the size of the image and draw the
    # watermark in that layer.
    layer = Image.new('RGBA', im.size, (0,0,0,0))
    if position == METHODS[1]:#tile
        for y in range(0, im.size[1], mark.size[1]):
            for x in range(0, im.size[0], mark.size[0]):
                layer.paste(mark, (x, y))
    elif position == METHODS[2]:#scale
        # scale, but preserve the aspect ratio
        ratio = min(
            float(im.size[0]) / mark.size[0], float(im.size[1]) / mark.size[1])
        w = int(mark.size[0] * ratio)
        h = int(mark.size[1] * ratio)
        mark = mark.resize((w, h))
        layer.paste(mark, ((im.size[0] - w) / 2, (im.size[1] - h) / 2))
    else:
        #by offset
        x, y        = position
        m_x, m_y    = mark.size
        l_x, l_y    = layer.size
        if x < 0: 
            x       = l_x-m_x+x
        if y < 0: 
            y       = l_y-m_y+y
        layer.paste(mark, (x,y))
        
    # composite the watermark with the layer
    return Image.composite(layer, im, layer)

class Action(models.Action):
    """Apply a watermark with tiling, scaling and opacity"""
    
    label       = _t('Watermark')
    author      = 'Stani'
    email       = 'spe.stani.be@gmail.com'
    version     = '0.1'
    tags        = [_t('filter')]
    __doc__     = _t('Apply a watermark with tiling, scaling and opacity')
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Mark')]              = models.ImageReadFileField(' ')
        fields[_t('Method')]            = models.ChoiceField(METHODS[0],
                                            METHODS)
        fields[_t('Horizontal Offset')] = models.PixelField('5')
        fields[_t('Vertical Offset')]   = models.PixelField('5')
        fields[_t('Opacity')]           = models.SliderField(60,1,100)
        
        super(Action,self).__init__(fields)
        
    def apply(self,photo,setting,cache):
        #get info
        info        = photo.get_info()
        #size
        x0, y0      = info['new_size']
        dpi         = info['new_dpi']
        #position
        method      = self.get_field_value('Method',info)
        if method == METHODS[0]:
            pos     = ( self.get_field_size('Horizontal Offset', info,x0,dpi),
                        self.get_field_size('Vertical Offset', info,x0,dpi))
        else:
            pos     = method
        #mark
        mark        = Image.open(self.get_field_value('Mark',info))
        opacity     = self.get_field_value('Opacity',info)/100.0
        if opacity < 1:
            mark    = reduce_opacity(mark, opacity)
        parameters  = {
            'mark'          : mark,
            'position'      : pos,
        }

        #manipulate layer
        return self.apply_to_current_layer_image(photo,watermark,**parameters) 
    
    def import_modules(self):
        #lazily import
        global Image, ImageEnhance
        import Image, ImageEnhance
        
    icon  = \
'x\xda\x01a\x0c\x9e\xf3\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\x00\
\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\x08\
\x08|\x08d\x88\x00\x00\x0c\x18IDATh\x81\xddXkl\\\xc7u\xfef\xee\xdc\xd7>\xee\
\xbe\xb8\x94\xb8\xa4)\x91\x94\x14[oQR`\x1b\n$\xc5\t\xa2\x04q\xa2 o$N\xf2\xa3\
)Z\x04)\xd0\x7fE\x9a\xa0(\xda\xfcmQ\x14P\xf2\'\x08\xfa\xcbF\x8b\xd8E\x1eF\
\x028q,9Jb\x92\x96lR\x94)\xc9|iW\xe4r\xdf\xaf\xbb\xf79\xd3\x1f\xcb]\xafX\x15\
\xda\xb5\xd6\x7fz\x80\x83{\xee.\xe6\xcc\xf9f\xbes\xe6\xcc%\xf8\x00\xe4C\x07\
\xf6\xb1\xc9\xc9I\x16\x8dF\x15*I\xb4i6\xddL&\xed\xce/\xcc\xbb\xa6i\x8bA\xceE\
\x06\xe5\xe8\xe4\xf44=|\xf8Ptjjjjxx\xd7I\xcb\xb2R\xe5r)*\x84P\xc2\x86Q\r\x87\
\xc2\x05\xc7q\xde\xbe\x9b^_z\xe7\xe6\xcd\xf4\xcf\x7f\xf1+g\x10\xf3\xb2A8\x99\
>~T\xfe\xc2\x17>\xff\x89C\x87\x8f|u\xcf\x9e\xf1\xe3\xc1`(\xb5\xb5\x95\xd5ggg\
\xa9eY\xe4\xd0\xa1\xc3|rr\xd2\x13B\x14r\xb9\xdc\xea\xc1\x83\x87~=\xf6\xd8c?\
\xb9t\xe9\xc7\xd9G\x9d\xfb\x91v\xe0{\xdf\xfb;6::zt\xcf\x9e\xbd\xdf}\xfc\xf1\
\'\xbe,\xcbL\xcfnf\xb1x\xf3&\x16\x17o \x9dN\xc3\xb6\x9a0"Q\x1c\xd8\x7f\x00\
\xc7\x8e\x1d\xc3\x9e\xbd{\x11\n\x05\xc5\xc6\xbd\x8d\xeb\xcb\xcb\xef\xfe\xfb\
\xca\xca\xca\x7f\xff\xfd\xf7\x7fPz\xbf1H\x8f\x02\xe0\xdb\xdf\xfe\x8b\x8fMO\
\x9f\xfc\xc7}\xfb\xf6\x7f\xaaZ\xadj\xe9\xf4]lll`ue\x05\xab++\xfc^&\xe3\x16\
\x8b%\xd7\xb6m\x10\x02\xa2\xa8\nql\x0bV\xb3Iv\x8f\xec\xde=2\x92:\xa5\xa8\x8a\
\x1a5\xc2s3\xb3s\xef\x8bR\xef\x1b\xc0\x0f\xff\xf9\x9f\x0e\x9c9\xf3\x91\x1f\
\xa5R\xa3\x1f^__SJ\xa52\x1c\xc7\xc1\xfc\xfc\xbc53;7\x97N\xa7\x7fY\xadV\x7f\
\xee\xb8\xce\xaf,\xcb\x9a\xa97\x1ai\xd7q\xa3CCC\xa1Db\x88\xa6\xd3wI$\x121\
\xc6\xc6\xc6Np!r\xba\xaa\xcc\xcf/\xdc\xf0\xfa\x8d\xa3\xef\x1cH\x8d\x0c\xe3\
\xaf\xfe\xfa;SO=\xf5\xf4\xbf$\x93\xc3\xd3o\xbe9\x87L:\x03\xcb\xb6J\xaf\xbf\
\xfe\x87W\x17\x16\x16~Z.\x97_\xb5\x1d\xc7\xcc\xe7r\x9d\x8a\x93L\x0e\xd3\xd5\
\xd5\xb5\xd4\xed;\xb7?}\xfe\xfc\xf9\xe7\xf6\xef\xdf\xff\xe1\xd7^{\x8d\x9d:u\
\xda8~\xfc\xc4\x0f\x9a\xa6\xd9\x10D<\xff\xfc\xf3\xff\xd9\xd7N\xf4\r\xe0\xc2\
\x85\x0b\xfa\xf8\xf8\xf8\xe7TM;\xf3\xe6\x9bs\xf8\xe3\x1f\xaf\xa2^oTn\xdd\xba\
\xfd\xaf\xab\xab\xab\xffQ,\x16\xd3\xb9\\\x8e\xef\x1c\x97\xcbm\xf1\\\x0e\xe9R\
\xa9\xf4\x13\xc7q\xffT,\x16\x7f(3\xf6\xcc\xe6\xe6\xa6z\xee\xdc\xf9\xe1X,\xfe\
\xadTj\xf4\r\x007?P\x00F$\xba\x97\x10\xf2\xb9|>o\xbc~\xe52n,\xdeh\xd6j\xf5\
\x9fnll\xfe\xdb\x9d;w\xaa\x0f\x1b_\xa9T\xdc\xabW\xaf\xbe\xa5\xa9\xea\xf7C\
\xa1\xa0B\x089\x1f\x08\x04\xa4\xb1\xd1\xd1\xd3Lb\x9f\xee\x17@\xdf9p\xfa\xd4\
\xa9\xafRJ\x9f3\x9b\xa6|\xf9\xf2kn\xb9\\\xf9Y\xb5R\xfd\x87w\x96\x96\n\xfd\
\xf8\xa9\xd5\x1bYUSV<\xd7}\xca\xb6\xed\xa4\xe7\xfb\x8aeYc\x96\xd5|as3\xdb\
\xec\xd5\x0f\xedg\xd2}S\x13\x8c1vVbL/\xe4\xf3\xb0m{\xd3\xf3\xfd\xff\xb2\x1dg\
\xab\x1f?\x00\x90\xcf\xe7\x84\xef\xf9s\x00y\xb9\xd1h\xa0^\xafC\x96\xe5\xc9\
\xe1\xe1\xe1c\xfd\xf8\xe9\x0b@0\x18\n1Y>\xae(\n\xca\xe52|\x9f\xdf\xf4}\xff\
\x8d\xdb\xb7o\xfb\xfd\x85\xdf\x92\x99\xd99\x13\xc0o<\xcf\xb3\x1d\xdb\x81\xc4\
\x98\xaa*\xea\x93\x81@\xb0gj\xf7\x05 \x9e\x88G\x15Y\x8e\xc9LF\xb3i\x82\x10r\
\xb5a6\xfb^\xfdnaLz\x8bs\xbf\xe8\xf9\x1e|\x9fCV\xe4#\xc3\xc3I\xad\xd7\xf1}\
\x01`\x12KQJuB\t\\\xd7\x83$\xd1\xbb\xb7\x96\x96\xfa\xae\xdd\xdd"\x84p\x84@\
\x05\x00\x04\xe7Pd%\xa2\xaa\xaa\xda\xeb\xf8\xbe\x00p\xce\r\x80\xc8\x10\x00\
\xe7\x1c\x02x\xe4\x86\x8cR\xea\x13\x02\x0b\x00\x04\x04\x00\x12\xa0\x84\xe8=\
\x8f\xefw>@\xc0\xe7>\x04\xc4@ZYB\x88\x00 \x80vc&(\xfa\xa8\x8e\xfd\x02\x10\
\x00 \x84\x80\x10\xa2\xf3>0!\xfd/I\xbf\x00\x88\x10\x02\x9c\xf3v\xe8\x03\xbaO\
\x10\x10B\xde\x97\xb3~\x01\x80\x10\x02\xc1\xc5\x80\x97\xbe\xe5\xf7\x03\xdf\
\x01\xce9|\xdf\x87\xeb8\xe0\xdco\xed\xc4#\x8a\xeb\xba\x84s\x0e!Z9\xd5\x9e\
\xa3W\xe9\xe9\xc08w\xee\\hbb\xe2\x99x<\xfe\xc9xb\x88\xea\xba\x86\xf1\xf1=\
\x98\x9c\x9c\xfa\xf8\xd7\x9f\xfbf\x8c\x10B$I\x82$I\xa0\x94\x821\xd6\xb1\t!\
\x9d\xa0v*\xe7\\8\x8e\xa3\xae\xac,\'\xa3\x91(\x82\xa1\x10\xa8$\xa5\x86\x92\
\xc9\xcf\xef\xdb\xbf\xff\x85\x97_~\xf9\xde@\x00$\x12\x89\xc4\xc4\xc4\xc4\xdf\
\x1c9r\xe4\xa3\xe3\xe3\xe3P\x14\x05\xa6i\xa2^\xaf\x7f\x03\xc07\x14E\x81\xaa\
\xaa\x90e\x19\x8c1\xc8\xb2\xdc\xb1)\xa5\xf0<\x0f\xae\xeb\xc2u\xdd\x8e\xed8\
\x0e\x1c\xc7\x81\xef\xfb8y\xf2$\x02\x81\x00$I\xc2\xd6\xd6\xd6\xd4\xe2\xe2\
\xe2\xdfV\xab\xb59\x00\x83\x01 I\x92`\x8c\tM\xd3\x10\x0c\x06A)E\xb3\xd9D\xa5\
R\x81eYP\x14\x05\x81@\x00\xaa\xaaBUUh\x9a\x866(J)\x1c\xc7\x81m\xdb\x1du\x1c\
\x07\xa6i\xa2uS#`\x8c!\x1c\x0ew\xc6\xca\xb2\x0cI\xea\xad\x92\xf6\x04 \x18\
\x0c\x92X,F\x92\xc9$b\xb1\x18\x08!\x9d\xe0\x8a\xc5"\x84\x10\xf7\xad8\xd0\xe2\
\xb2\xe7y\xa0\x94\x82s\x0eB\x08(\xa5\x90e\xb9c\x07\x83Ah\x9a\x86\x91\x91\x11\
\x84B\xa1\x0e\xf5666\xa0\xeb\xbd\x9de\xbd\x02@8\x1c\x86a\x18\xd04\xadU\x89\
\x84@8\x1c\xee\xd8B\x08PJ\xd1\xce\x85\xb6\x12B:\x01K\x92t_\x0e\x10B\xa0\xeb:\
\x02\x81\x004M\x03c\x0c\xbe\xef\xc30\x8c\xc1\x02(\x95J\xc8\xe5r(\x14\n\x08\
\x04\x02 \x84\xa0^\xaf#\x93\xc9\xc04M\xc8\xb2\xdcIXY\x96;4PU\x15\x84\x10x\
\x9e\xd7\xa1Q;\x17l\xdb\x06\xe7\x1c\x9a\xa6A\x08\x81X,\x06Y\x96Q\xab\xd5P(\
\x14P\xad>\xf4n\xd4;\x80vb\xb6)\xd2\xbd\xea\xedmo\xafv\xf7\xaa\x03\xb8\xef)I\
\x128\xe7\xe0\x9cC\x96\xe5N\x19n?\xdb\xf4k\xfb\x1b\x18\x00\xc30\x10\x8b\xc5\
\x10\x0e\x87;[kY\x16"\x91H\x87\xe7m\x9a\xb4A\xee\xa4\x90$I\x90e\xf9>\n\t!:4j\
\'\xb0m\xdb\x08\x87\xc3\x08\x04\x02\x83\x03P*\x95\x90\xcf\xe7Q\xa9T\x10\n\
\x85\xc09G\xb1XD>\x9f\x87\xe7\xb5\xba\xe9\xf6.)\x8a\xd2\xd1v5\xe9.\x9bm\n\
\xb5K(\xa5\x14\xbe\xefw\xf8\xdfh4P\xa9T\xd0h4z\x02\xd0\xf3I\xdc^\xc9\x07\xd1\
\xa3\xfb\xff\xb6l7{\x9d\'y@\x9b\xd0\xed\xafM\xc9\x9d~\x1e&=\xed@$\x12A$\x12A\
 \x10\x80\xa2(\xe0\x9c#\x14\n\xc1u\x9dN-\xa7\x94\x82R\t\x92DA)\x81\x10\x1c\
\x8e\xe3\x82R\xd2\xa1L\xbb\x8cRJ\xa1\xaa2 \x04\x08\xa1\x08\x06u(\x8a\xbc\xad\
\x0c\xba\xae\xa1\xd7;MO\x00\xae_\xbf\x8e\\.G2\x99\x0c\xa6\xa6\xa6P(\x14\xb0\
\xbe\xbe\x0e\xcb\xaa#i40\xbe\xcb\x83\xa6\x02\x94\x10H\x12\x01\x93\x08@(\xee\
\xe5\x18\xae\xdf\x120\x9b\x1e8\xef*\x9f\xe08r\x80\xe2\xe0\xa4\x8c`@\x81\x9fW\
a\x87\xa2\x08G\xf7\xc0\xb6}\xdcz\xfb:\xb91\xbf\xd0\x13;z\x02\xb0\xb6\xb6\x86\
\xd5\xd55\xe4\xf3\x05d\xb3Y\xdc\xb9s\x07KKK\xb0\x9a\r\xc4\x825<{\x96\xe2k\
\x17<\x0c\xc7}P\xda\xe2\xa5\x00P\x8aH(\xae\x12\xbc\xf0\xaa\x8f[k\x1c\xae\xf7\
^\x0f;\x9b\xa2\xf8\xd2\x85\x08\xbe\xf2\xc9\x00\xf6\x8cp\x10\xaaC\n\x1e\x87\
\xad\x05 Y\xd7\xc8\xd6f\xb6\'\x1e\xf5T\xab\xa2Q#:\x92\xf0/>>\x15\x9d\xd0\x82\
\t\xac\xad\xa7\x91/\x14P\xa9Tqw\xa3\x8e\xd7\xaf9\xf8\xdd\x0cA\xa5N1\x14\xe1\
\xd8\x9d\xf0`\xe8\x1e\xe2a\x17O\x1fs\xf1\x99\x8fp\xecM\tl\xe4}Tj>,\x9b\xa3X\
\xe1\xb8~\x8b\xe2\xea\xdb\n\x08\x08\x12\x86\x8dX,\x0eY\xd6\xf0\xd6B\xa66\xbb\
\xd0xi\xab\xd0X\x19\x08\x80D"\x16}\xe64\xbf\xf8\x97_\x8cL\x8c\x8f&\xd1\xa8\
\xdeC\xaeP\xc1\xe6V\r\x9e\xd7j}\xf3%\x1fo,p\\\xb9NQ\xac0(*E,\x0ch\x8a\x80\
\x11\x108\xf1!\x81\xa7\x8fQ\xa4\x92\x04\xb6C\x90-\x00\x84\xcah\xba\nf\x16|\
\xcc\xdcp\xc0\xd4a\x8c\xec\x0e\xe3\xe6\xad{\xf5\xb9\xc5\xe6\x8b[\xf9\xfa`\
\x00LMND\x0fMy\x17?\xfadp\xe2\x89\xa9\x04NLfpb_\x15\xaeSE\xa1,Po\x02B\x00\
\xbe/\x90+q\xcc.\n\\\xb9F\xb0|OAjXB\xdc\xe0\x90\x190\x1c\x13\x98~\x028;\r$"\
\x14\xe5\xba\x82\x86\xa5\xc2\xb29\x96\xd7Ml\xd5\xe20\x9d(\xe6\xdf\xc9\x9a\
\x9by\xf1Rv\xab\xbc<\x10\x00cccQ\x99\x98\x17SIebd\xf7\x10"j\x1a\x93\xbb\xcbx\
\xf6\x8c\x89\xcf\x9e\x05\x0e\x8c\x03\xb5\x06`9\x80\xed\x00\xb6+\x90/q\xcc,x\
\xf8\xd9o\t\xe6\xdfUA\x08\xc5PT \xa4\x0b$\xa3\xc0\x99i\xe0\xd9\xb32&\x1f\xd3\
\xb0U\xf4\x91+4\xa1h\x118"\x81\x99k\x99F\xb3\xe9\xbdX.\x0f\x08\x80\xa6i\xd1;\
\xeb\x8d\x8b\x8b\xab\xca\x84Ob\xc8\xe7\xeeA\x93\xca\x88\x04\x1d\x0cE\x81\x93\
\x07\x81s\xa7\x80\xc3S\x80\x11"(\xd7\x08\x1aM\x80s\xc0\xb48\x16\x97}\xfc~\
\x16x7-\xc1\xf5$D\xc3\x80\x11\x14\x08\x07\x19\x0e\xed\xd7p\xfa\x90\x87T\xdc\
\x82$\x87\xe0\x88\x18\xd6\xd6\xd3\xf5Z\xbd\xfeb\xb3\xd9\x1c\x0c\x85v\xed\xda\
\x15%\x94]\xa4\xcc\x98(\xd4Bx\xf5\x8d\x1a.\xcf\x9a\xa87<\x08!\xe0\xba\x80\
\xa6\x00{G\x80\xd3\x07\x81#\xfb(BA\x86\xad"E\xad\xd1:\xa0\x1aM\x81\xc5e\x81+\
\xd7\x04\xfe\xbc\xd0*\xb7\x00C\xae\xa2\xa3\xd2`\xd05\x82D<\x86\xaa\x15G6g\
\xd6<\xcf\x7f\xa9Z\xad>\x14@Oe\xf4\xfc\xf9\xf3\x98\x9e\x9e\x16G\x8f\x1e\x85a\
\x18\xa8T*H\xa73\xc8d\xd2\xb8\xbcY\x05\xd9p \xb8\x0f@\x00\x04\xa0L\xe0\xe8\
\x19\x82\xbd\'\x08\x1a\xcdV\xa3v\xdf\tN)\x96}\x19\xa5M\x15\x86\x11\x85\x11M"\
2\x1a\xc7x|\x08O2\t\xcf,\xbd\xe3\xbf\xf2\xca+\xde\xa5K\x97\x1e\x1a[O\x00\\\
\xd7\xed\xf4\xfb\xedFK\xd7u\xc4\xe3\xb1\xd6\'F&\x81I\xac\xd5\x99v\xd9\xad\
\xc6\xee\xbd\x93\xd8\xf3\xbc\x8e\xed\xba.|\xce\xc1$\t\xa1P\x18\x91H\xeb\xaeQ\
\xaf\xd7;\xedG/\xd2\x13\x80j\xb5\x8aB\xa1\x80|>\x0fEQ\xe0\xfb~\xe7\x8e\xe0y\
\xde}\r\x9c\xa2(\xdb=\xbe\x02B\x00!h\xa7\x91k_\'\xdb\xb6\xe7y`\x8c\xa1\xd9l\
\xc2u\x1d\xe8\xba\x8ej\xb5\x8ab\xb1\x08\xd34\x07\x06\x80\xb8\xae+l\xdb\xe6\
\x95J\x85s\xceE\xbd^G6\x9b\xc5\xe6\xe6&\x1c\xc7\xd9\xee\x83\xdek\xa1\xdbw\
\x04\xc6\x18\x08!\x9d\xd5\xef\xde\x81\xf6\xa7\x14]\xd7a\x18\x06\x92\xc9$\xa2\
\xd1(\x00\x10\xd34\xb9\xe3\xf4\xf6\xd9\xf5A\x00H\x97R\x00tmm\xcd\xd5u\xfdJ\
\xa1P(\xf9\xbe\x8fZ\xadFL\xd3\x84i\x9a\x84sN\xb6\x83\'\x94R\xb2}\' mP\xdd\
\x9fU8\xe7\xa2\x1d\xbc\xef\xfb\x82s.TU\x85$IB\xd34\xa1\xeb\xba\x88D"\xa8\xd5\
j\x85\xb5\xb5\xb5\n\x00\x05\xad\xaeD\x00\xe0x\xefS\xa6\xe8\x0e\xb6[h\x97Jm[Q\
\x14644\x14\x92eY#\x84\x10\xcf\xf3$\xce9\xe5\x9cS!DG\xbb\xc6\x92\xedw\xb2\
\x9d\xb8;\x83\xe0\x84\x10\x0e\x803\xc6\xf8\xf6\xbbO)\xe5\xb2,s\xcf\xf3\xbcB\
\xa1Pk4\x1a\x0e\x80\xed\xea\x00\x7f[\xf9\xb6>\x10@\xf7\x0e\xd0\xee]x\x80\x92\
\x07\xd8d\x87\xee\x94\xff\x05\xa4\xeb\xc9\xff\x8f\xf7\x9d\xbf\xdf\x97\xe1\
\xbdt|\xe4\x01\xcf\x87\xd9\x0f\xf2\xbd\xb3\xb4\xf0\x1d\xffu\xeb\xce\xdf\xff\
\xff\xca\xff\x00\x90j3\x81M\xd7)\xad\x00\x00\x00\x00IEND\xaeB`\x82\x97\xbc\
\xff\xdc'
