#license: latest GPL

#---import modules

#standard library
import cPickle, os, pprint, time, traceback

#gui-independent
from core import ct, pil
import formField
from core.lib.events import send

#---classes
class PathError(Exception):
    def __init__(self, filename):
        """PathError for invalid path."""
        self.filename   = filename
        
    def __str__(self):
        return '"%s" '%self.filename+_('is not a valid path')+'.'
 
class ReadOnlyDict:
    def __init__(self,data):
        self.data   = data
        
    def __call__(self,key):
        return self.data[key]

#---init/exit
def init():
    create_settings()
    verify_app_user_path()
    
def exit():
    pass
        
def create_settings():
    global SETTINGS
    SETTINGS    = {
        'path'                      : ct.USER_PATH,
        'extensions'                : formField.IMAGE_EXTENSIONS,
        'recursive'                 : False,
        'description'               : True,
        'stop_for_errors'           : True,
        'create_missing_folders'    : True,
        'overwrite_existing_images' : False,
        'check_images_first'        : True,
        'file_history'              : [],
    }
    if os.path.exists(ct.SETTINGS_PATH):
        f       = open(ct.SETTINGS_PATH,'rb')
        SETTINGS.update(cPickle.load(f))
        f.close()

def verify_app_user_path():
    if not os.path.exists(ct.APP_USER_PATH):
        os.mkdir(ct.APP_USER_PATH)

#---error logs
def init_error_log_file():
    global ERROR_LOG_FILE, ERROR_LOG_COUNTER
    ERROR_LOG_COUNTER   = 0
    ERROR_LOG_FILE      = open(ct.ERROR_LOG_PATH, 'w')

def log_error(message,details,filename,action=None):
    global ERROR_LOG_COUNTER
    details = os.linesep.join([
        'File: "%s"'%filename,
        'Details: "%s"'%details,
    ])
    if action:
        details += os.linesep + 'Action: ' + pprint.pformat(action.dump())
    ERROR_LOG_FILE.write(os.linesep.join([
        'Error %d: %s'%(ERROR_LOG_COUNTER,message),
        details,
        os.linesep,
    ]))
    traceback.print_exc(file=ERROR_LOG_FILE)
    ERROR_LOG_FILE.write('*'+os.linesep)
    ERROR_LOG_FILE.flush()
    ERROR_LOG_COUNTER += 1
    return details

#---collect image files
def filter_image_files(folder,files, extensions):
    files   = [os.path.join(folder,file) for file in files]
    return [file for file in files if os.path.isfile(file) and \
        os.path.splitext(file)[1].lower() in extensions]
       
def get_image_files_folder(folder,extensions,recursive):
    if recursive:
        image_files  = []
        for folder, dirs, files in os.walk(folder):
            image_files.extend(filter_image_files(folder, files, extensions))
        return image_files
    else:
        return filter_image_files(folder, os.listdir(folder), extensions)
    

#---check
def check_actionlist(actions):
    #Check if there is something to do
    if actions == []:
        send.error('%s %s'%(_('Nothing to do.'),
            _('The action list is empty.')))
        return None
    
    #Skip disabled actions
    actions = [action for action in actions if action.is_enabled()]
    if actions == []:
        send.error('%s %s'%(_('Nothing to do.'),
            _('There is no action enabled.')))
        return None
    
    #Check if there is a save statement
    last_action = actions[-1]
    if not last_action.is_valid_last_action():
        send.frame_append_save_action()
        return None
    return actions

def check_images(image_files):
    #show dialog
    send.frame_show_progress(   title   = _("Checking images"),
                                maximum = len(image_files))
                                
    #check files
    valid = []; invalid = []
    for index, image_file in enumerate(image_files):
        pil.report_invalid_image(image_file,valid,invalid)
        result              = {}
        send.progress_update_filename(result,index,image_file)
        if not result['keepgoing']:
            return
    send.progress_close()
    
    #show invalid messages
    if invalid:
        result      = {}
        send.frame_show_files_message(result,
            message = _('Phatch can not handle %d image(s):')%len(invalid), 
            title   = ct.FRAME_TITLE%('',_('Invalid images')), 
            files   = invalid)
        if result['cancel']:
            return
    image_files              = valid

    #Check if there are files
    if image_files:
        return image_files
    else:
        send.frame_show_error('%s, %s.'%(_("Sorry"),_("no valid files found")))
        return

#---get
def get_image_files(path,extensions,recursive):
    try:
        result  = []
        for p in path.split(ct.PATH_DELIMITER):
            p   = p.strip()
            if os.path.isfile(p):
                result.append(p)
            elif os.path.isdir(p):
                result.extend(get_image_files_folder(p,extensions,recursive))
            else:
                raise PathError(p)
        return result
    except PathError, error:
        send.frame_show_error('%s, "%s" %s.'%(_('Sorry'),error.filename,
                _('is not a valid path')))
        return []

def get_path(path, settings,last_action,drop=False):
    if drop or (path is None):
        result  = {}
        send.frame_show_execute_dialog(result,settings,last_action,path)
        if result['cancel']:return
        path    = settings['path'].strip() 
        if not path:
            send.frame_show_error(_('No files or folder selected.'))
            return None
    return path

def get_photo(image_file,stop_for_errors):
    try:
        photo   = pil.Photo(image_file)
        return photo, False, False
    except Exception, details:
        photo   = None
        #log error details
        message =  '%s "%s".'%(_('Unable to open file'),image_file)
        ignore  = False
        action  = None
        return process_error(photo,message,details,image_file,action,
            stop_for_errors,ignore)
    
#---apply
def process_error(photo,message,details,image_file,action,stop_for_errors,ignore):
    """Logs error to file and show dialog box allowing the user to 
    skip,abort or ignore."""
    global last_answer
    details = log_error(message,details,image_file,action)
    #show error dialog
    if stop_for_errors:
        result  = {}
        send.frame_show_progress_error(result,message,details,
            ignore = ignore)
        #if result:
        answer, stop_for_errors = result['answer'],\
            result['stop_for_errors']
        if answer == 'abort':
            send.progress_close()
            return photo, False, True #photo, do_break, do_return
        last_answer = answer
        if answer == 'skip':
            return photo, True, False #photo, do_break, do_return
    elif last_answer == 'skip':
        return photo, True, False #photo, do_break, do_return
    return photo, False, False #photo, do_break, do_return

def apply_action(action,photo,setting,cache,image_file,stop_for_errors):
    try:
        photo   = action.apply(photo,setting,cache) 
        return photo, False, False #photo, do_break, do_return
    except Exception, details:
        message = '%s "%s".'%(_('Can not apply action'),_(action.label))
        ignore  = True
        return process_error(photo,message,details,image_file,action,
            stop_for_errors,ignore)
    
def apply_actions(actions,settings,path=None,drop=False):
    """Do all the actions."""
    
    #Start log file
    init_error_log_file()
    
    #Check action list
    actions = check_actionlist(actions)
    if not actions: return
    
    #Get path (and update settings)
    path    = get_path(path,settings,last_action=actions[-1],drop=drop)
    if not path: return
            
    #Handle settings - extract often used values
    stop_for_errors = settings['stop_for_errors']
    extensions      = ['.'+x for x in settings['extensions']]
    
    #Check if all files exist
    image_files     = get_image_files(path,extensions,settings['recursive'])
    if not image_files: return
    
    #Check if all the images are valid
    if settings['check_images_first']:
        image_files = check_images(image_files)
    if not image_files: return

    #Initialize actions
    for action in actions:
        action.import_modules()
        
    #Retrieve settings
    skip_existing_images    = not settings['overwrite_existing_images']
    stop_for_errors         = settings['stop_for_errors'] 
    
    #Execute action list
    global last_answer
    actions_amount          = len(actions)
    cache                   = {} 
    is_done                 = actions[-1].is_done #checking method for resuming
    last_answer             = None
    read_only_settings      = ReadOnlyDict(settings)
    
    #Start progress dialog
    send.frame_show_progress(   title   = _("Executing action list"),
                                maximum = (actions_amount+1)*len(image_files))
    send.progress_set_index(actions_amount)
    
    for image_index, image_file in enumerate(image_files):        
        #update image file & progress dialog box
        result              = {}
        send.progress_update_filename(result,image_index,image_file)
        if result and not result['keepgoing']:
            send.progress_close
            return
        time.sleep(0.001)
        
        #open image and check for errors
        photo, do_break, do_return = get_photo(image_file,stop_for_errors)
        if      do_return:  return
        elif    do_break:   break
        
        #check if already not done
        if skip_existing_images and is_done(photo):
            break
        
        #do the actions
        for action_index, action in enumerate(actions):
            photo, do_break, do_return  = apply_action(action,photo,
                                    read_only_settings, cache,image_file,
                                    stop_for_errors)
            if      do_return: return
            elif    do_break:
                break
            result  = {}
            send.progress_update_index(result,image_index,action_index)
            if result and not result['keepgoing']:
                send.progress_close()
                return

    send.progress_close()
