#license: latest GPL
"""
Store internally as a string.
Provide validation routines.
"""

#---import modules

#standard library
import os

if '_' not in dir():
    _ = str

#gui independent (core.lib)
from odict import odict as Fields

NO_FIELDS               = Fields()
_t                      = unicode
#---image
ALIGN_HORIZONTAL        = [_t('left'),_t('center'),_t('right')]
ALIGN_VERTICAL          = [_t('top'),_t('middle'),_t('bottom')]

IMAGE_EXTENSIONS        = ['bmp','dib','gif','jpe','jpeg','jpg','im','msp',
                        'pcx','png','pbm','pgm','ppm','tif','tiff','xbm']
IMAGE_READ_EXTENSIONS   = IMAGE_EXTENSIONS + ['cur','dcx','fli','flc','fpx',
                        'gbr','gd','ico','imt','mic','mcidas','pcd',
                        'psd','bw','rgb','cmyk','sun','tga','xpm']
IMAGE_WRITE_EXTENSIONS  = IMAGE_EXTENSIONS + ['eps','ps','pdf']
IMAGE_MODES             = [_t('Monochrome (1-bit pixels, black and white)'),
                        _t('Grayscale (8-bit pixels, black and white)'),
                        _t('RGB (3x8-bit pixels, true colour)'),
                        _t('RGBA (4x8-bit pixels, RGB with transparency mask)'),
                        _t('CMYK (4x8-bit pixels, colour separation)'),
                        _t('P (8-bit pixels, mapped using a colour palette)'),
                        _t('YCbCr (3x8-bit pixels, colour video format)'),
                        _t('I (32-bit integer pixels)'),
                        _t('F (32-bit floating point pixels)')]
IMAGE_RESAMPLE_FILTERS  = [_t('nearest'),_t('bilinear'),_t('bicubic'),
                        _t('antialias')]

IMAGE_MODELS_WRITE_EXTENSIONS = ['<type>']+IMAGE_WRITE_EXTENSIONS

IMAGE_READ_EXTENSIONS.sort()
IMAGE_WRITE_EXTENSIONS.sort()

#---os
def ensure_path(path):
    if not os.path.exists(path):
        ensure_path(os.path.dirname(path))
        os.mkdir(path)

#---Form
class Form(object):
    #todo: move this as instance attributes
    label   = 'label'
    icon    = 'ART_TIP'
    tags    = []
    __doc__ = ''
    
    def __init__(self, fields = NO_FIELDS):
        """For the parameters a Gui needs to be written"""
        #self.label              = label do this....
        fields['__enabled__']   = BooleanField(True,visible=False)
        self._fields            = fields
        
    def __cmp__(self, other):
        label       = _(self.label)
        other_label = _(other_label)
        if label < other_label: return -1
        elif label == other_label: return 0
        else: return 1
        
    def get_fields(self):
        return self._fields
    
    def get_field(self,label):
        return self._fields[label]
    
    def get_field_value(self,label,info):
        return self.get_field(label).get(info)
    
    def get_field_size(self,label,info,base,dpi):
        return self.get_field(label).get_size(info,base,dpi)
    
    def get_field_string(self,label):
        return self.get_field(label).get_as_string()
    
    def is_enabled(self):
        return self.get_field_value('__enabled__',None)
    
    def set_field(self,label,value):
        self._fields[label].set(value)
        return self
        
    def set_field_as_string(self,label,value_as_string):
        self._fields[label].set_as_string(value_as_string)
        return self
        
    def load(self,fields):
        for label, value in fields.items():
            self.set_field_as_string(label,value)
        return self
            
    def dump(self):
        fields  = {}
        for label, field in self._fields.items():
            fields[label]  = field.value_as_string
        return {'label':self.label,'fields':fields}
    
class ValidationError(Exception):
    
    def __init__(self, expected, message, details=None):
        """ValidationError for invalid input.
        
        expected - description of the expected value
        message  - message why validation failed
        details  - eg. which variables are allowed"""
        self.expected       = expected
        self.message        = message
        self.details        = details
        
    def __str__(self):
        return self.message
    
class Field(object):
    """Base class for fields. This will be subclassed but, 
    never used directly.
    
    Required to overwrite:
    description - describes the expected value
    
    Fieldal to overwrite
    to_python   - raise here exceptions in case of validation errors (defaults 
                  to string)
    to_string   - (defaults to string)
    
    Never overwrite:
    validate    - will work right out of the box as exceptions are raised by
                  the to_python method
    get         - gets the current value as a string
    set         - sets the current value as a string
    
    You can access the value by self.value_as_string
    
    This field interpolates <variables> within a info.
    << or >> will be interpolated as < or >
    """
    
    description             = '<?>'
    
    def __init__(self,value,visible=True):
        self.visible    = visible
        if isinstance(value, (str, unicode)):
            self.set_as_string(value)
        else:
            self.set(value)
        
    def interpolate(self,x,info):
        if info == None:
            return self.value_as_string
        else:
            try:
                return x.replace('%','%%')\
                    .replace('<','%(').replace('>',')s')\
                    .replace('%(%(','<').replace(')s)s','>')%info
            except KeyError, variable:
                raise ValidationError(self.description,
                "%s '%s' %s."%(_("The variable"),
                    variable,_("doesn't exist")),
                "%s: %s."%(_('The defined variables are'),
                    (', '.join(info.keys()))))

    def to_python(self,x):
        return x
    
    def to_string(self,x):
        return unicode(x)
    
    def get_as_string(self):
        """For GUI: Translation, but no interpolation here"""
        return self.value_as_string
    
    def set_as_string(self,x):
        """For GUI: Translation, but no interpolation here"""
        self.value_as_string    = x
        
    def get(self,info=None,value_as_string=None):
        """For code: Interpolated, but not translated
        - value_as_string can be fieldally provided to test the expression"""
        if value_as_string is None:
            value_as_string = self.value_as_string
        return self.to_python(self.interpolate(value_as_string,info))
   
    def set(self,x):
        """For code: Interpolated, but not translated"""
        self.value_as_string  = self.to_string(x)
        
    def eval(self,x):
        try:
            return eval(x)
        except SyntaxError:
            pass
        except NameError:
            pass
        raise ValidationError(self.description, 
            '%s "%s" %s.'%(_('Invalid syntax'),x,_('for integer')))
        
class IntegerField(Field):
    """"""
    description             = _('integer')
    
    def to_python(self,x):
        error   = ValidationError(self.description, 
            '%s "%s" %s.'%(_('Invalid literal'),x,_('for integer')))
        try:
            return int(round(self.eval(x)))
        except ValueError:
            raise error
        except TypeError:
            raise error

class PositiveIntegerField(IntegerField):
    """"""
    description = _('positive integer')

    def to_python(self,x):
        value = super(PositiveIntegerField, self).to_python(x)
        if value < 0:
            raise ValidationError(self.description,
            '%s "%s" %s.'%(('The integer value'),x,
                _('is negative, but should be positive')))
        return value
    
class PositiveNonZeroIntegerField(PositiveIntegerField):
    """"""
    
    description = _('positive, non-zero integer')

    def to_python(self,x):
        value = super(PositiveNonZeroIntegerField, self).to_python(x)
        if value == 0:
            raise ValidationError(self.description,
                '%s "%s" %s.'%(_('The integer value'),x,
                    _('is zero, but should be non-zero.')))
        return value
                
class DpiField(PositiveNonZeroIntegerField):
    """PIL defines the resolution in two dimensions as a tuple (x,y).
    Phatch ignores this possibility and simplifies by using only one resolution
    """
    
    description = _('resolution')
    
class FloatField(Field):
    description             = _('float')
    
    def to_python(self,x):
        try:
            return float(self.eval(x))
        except ValueError, message:
            raise ValidationError(self.description, 
            '%s "%s" %s.'%(_('Invalid literal'),x,_('for float')))

class BooleanField(Field):
    description             = _('boolean')
    
    def to_string(self,x):
        return super(BooleanField,self).to_string(x).lower()
    
    def to_python(self,x):
        if x.lower() in ['1','true','yes']: return True
        if x.lower() in ['0','false','no']: return False
        raise ValidationError(self.description, 
            '%s "%s" %s (%s,%s).'%(_('Invalid literal'), x,_('for boolean'),
                _('true'),_('false')))
    
class CharField(Field):
    description             = _('string')
    pass
    
class ChoiceField(CharField):
    description             = _('choice')
    def __init__(self,value,choices,**keyw):
        super(ChoiceField,self).__init__(value,**keyw)
        self.choices    = choices
        
class FileField(CharField):
    extensions  = []

class ImageReadFileField(FileField):
    extensions  = IMAGE_READ_EXTENSIONS
    
class FileNameField(CharField):
    """Without extension"""
    pass
    
class FilePathField(CharField):
    pass
    
class ImageReadFileField(FileField):
    extensions  = IMAGE_READ_EXTENSIONS
    
class ImageTypeField(ChoiceField):
    def __init__(self,value,**keyw):
        super(ImageTypeField,self).__init__(value,IMAGE_EXTENSIONS,**keyw)
        
    def set_as_string(self,x):
        #ignore translation
        if x and x[0]=='.':
            x = x[1:]
        super(ImageTypeField,self).set_as_string(x)
        
##class ImageTypeField(ChoiceField):
##    def set_as_string(self,x):
##        if x and x[0]=='.':
##            x = x[1:]
##        super(ImageTypeField,self).set_as_string(x)
##        
class ImageReadTypeField(ChoiceField):
    def __init__(self,value,**keyw):
        super(ImageReadTypeField,self).__init__(\
            value,IMAGE_READ_EXTENSIONS,**keyw)
        
class ImageWriteTypeField(ChoiceField):
    def __init__(self,value,**keyw):
        super(ImageWriteTypeField,self).__init__(\
            value,IMAGE_MODELS_WRITE_EXTENSIONS,**keyw)
        
class ImageModeField(ChoiceField):
    def __init__(self,value,**keyw):
        super(ImageModeField,self).__init__(value,IMAGE_MODES,**keyw)
        
    def to_python(self,x):
        return x.split(' ')[0].replace('Grayscale','L').replace('Monochrome','1')
        
class ImageResampleField(ChoiceField):
    def __init__(self,value,**keyw):
        super(ImageResampleField,self).__init__(\
            value,IMAGE_RESAMPLE_FILTERS,**keyw)
            
    def to_python(self,x):
        return x.upper()
            
class AlignHorizontalField(ChoiceField):
    def __init__(self,value,**keyw):
        super(AlignHorizontalField,self).__init__(\
            value,ALIGN_HORIZONTAL,**keyw)
            
class AlignVerticalField(ChoiceField):
    def __init__(self,value,**keyw):
        super(AlignVerticalField,self).__init__(\
            value,ALIGN_VERTICAL,**keyw)
            
class PixelField(IntegerField):
    """Can be pixels, cm, inch, %."""
    def get_size(self,info,base,dpi,value_as_string=None):
        if value_as_string is None:
            value_as_string = self.value_as_string
        for measure, value in self._metrics(base,dpi).items():
            value_as_string = value_as_string.replace(measure,value)
        return super(PixelField,self).get(info,value_as_string)
        
    def _metrics(self,base,dpi):
        return {
            'cm'    : '*%f'%(dpi/2.54),
            'mm'    : '*%f'%(dpi/25.4),
            'inch'  : '*%f'%dpi,
            '%'     : '*%f'%(base/100.0),
            'px'    : '',
        }

class SliderField(IntegerField):
    """A value with boundaries set by a slider."""
    def __init__(self,value,minValue,maxValue,**keyw):
        super(SliderField,self).__init__(value,**keyw)
        self.min    = minValue
        self.max    = maxValue
            
class ColourField(CharField):
    pass
##    def to_python(self,x):
##        return eval(x.replace('#','0x'))
    
#todo
##class CommaSeparatedIntegerField(CharField):
##    """Not implemented yet."""
##    pass
##    
##class DateField(Field):
##    """Not implemented yet."""
##    pass
##    
##class DateTimeField(DateField):
##    """Not implemented yet."""
##    pass
##    
##class EmailField(CharField):
##    """Not implemented yet."""
##    pass
##    
##class UrlField(CharField):
##    """Not implemented yet."""
##    pass
